@@ -4,10 +4,12 @@ import (
 	"bufio"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"log"
 	"log/slog"
+	"maps"
 	"net"
 	"os"
 	"path/filepath"
@@ -19,6 +21,7 @@ import (
 
 	"github.com/mjl-/bstore"
 
+	"github.com/mjl-/mox/config"
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/message"
 	"github.com/mjl-/mox/metrics"
@@ -27,6 +30,7 @@ import (
 	"github.com/mjl-/mox/queue"
 	"github.com/mjl-/mox/smtp"
 	"github.com/mjl-/mox/store"
+	"github.com/mjl-/mox/webapi"
 )
 
 // ctl represents a connection to the ctl unix domain socket of a running mox instance.
@@ -294,6 +298,13 @@ func servectl(ctx context.Context, log mlog.Log, conn net.Conn, shutdown func())
 	}
 }
 
+func xparseJSON(ctl *ctl, s string, v any) {
+	dec := json.NewDecoder(strings.NewReader(s))
+	dec.DisallowUnknownFields()
+	err := dec.Decode(v)
+	ctl.xcheck(err, "parsing from ctl as json")
+}
+
 func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 	log := ctl.log
 	cmd := ctl.xread()
@@ -315,10 +326,10 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		*/
 
 		to := ctl.xread()
-		a, addr, err := store.OpenEmail(ctl.log, to)
+		a, addr, err := store.OpenEmail(log, to)
 		ctl.xcheck(err, "lookup destination address")
 
-		msgFile, err := store.CreateMessageTemp(ctl.log, "ctl-deliver")
+		msgFile, err := store.CreateMessageTemp(log, "ctl-deliver")
 		ctl.xcheck(err, "creating temporary message file")
 		defer store.CloseRemoveTempFile(log, msgFile, "deliver message")
 		mw := message.NewWriter(msgFile)
@@ -328,13 +339,13 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		err = msgFile.Sync()
 		ctl.xcheck(err, "syncing message to storage")
 
-		m := &store.Message{
+		m := store.Message{
 			Received: time.Now(),
 			Size:     mw.Size,
 		}
 
 		a.WithWLock(func() {
-			err := a.DeliverDestination(log, addr, m, msgFile)
+			err := a.DeliverDestination(log, addr, &m, msgFile)
 			ctl.xcheck(err, "delivering message")
 			log.Info("message delivered through ctl", slog.Any("to", to))
 		})
@@ -354,7 +365,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		account := ctl.xread()
 		pw := ctl.xread()
 
-		acc, err := store.OpenAccount(ctl.log, account)
+		acc, err := store.OpenAccount(log, account)
 		ctl.xcheck(err, "open account")
 		defer func() {
 			if acc != nil {
@@ -363,90 +374,227 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 			}
 		}()
 
-		err = acc.SetPassword(ctl.log, pw)
+		err = acc.SetPassword(log, pw)
 		ctl.xcheck(err, "setting password")
 		err = acc.Close()
 		ctl.xcheck(err, "closing account")
 		acc = nil
 		ctl.xwriteok()
 
-	case "queue":
+	case "queueholdruleslist":
 		/* protocol:
-		> "queue"
+		> "queueholdruleslist"
 		< "ok"
 		< stream
 		*/
-		qmsgs, err := queue.List(ctx)
+		l, err := queue.HoldRuleList(ctx)
+		ctl.xcheck(err, "listing hold rules")
+		ctl.xwriteok()
+		xw := ctl.writer()
+		fmt.Fprintln(xw, "hold rules:")
+		for _, hr := range l {
+			var elems []string
+			if hr.Account != "" {
+				elems = append(elems, fmt.Sprintf("account %q", hr.Account))
+			}
+			var zerodom dns.Domain
+			if hr.SenderDomain != zerodom {
+				elems = append(elems, fmt.Sprintf("sender domain %q", hr.SenderDomain.Name()))
+			}
+			if hr.RecipientDomain != zerodom {
+				elems = append(elems, fmt.Sprintf("sender domain %q", hr.RecipientDomain.Name()))
+			}
+			if len(elems) == 0 {
+				fmt.Fprintf(xw, "id %d: all messages\n", hr.ID)
+			} else {
+				fmt.Fprintf(xw, "id %d: %s\n", hr.ID, strings.Join(elems, ", "))
+			}
+		}
+		if len(l) == 0 {
+			fmt.Fprint(xw, "(none)\n")
+		}
+		xw.xclose()
+
+	case "queueholdrulesadd":
+		/* protocol:
+		> "queueholdrulesadd"
+		> account
+		> senderdomainstr
+		> recipientdomainstr
+		< "ok" or error
+		*/
+		var hr queue.HoldRule
+		hr.Account = ctl.xread()
+		senderdomstr := ctl.xread()
+		rcptdomstr := ctl.xread()
+		var err error
+		hr.SenderDomain, err = dns.ParseDomain(senderdomstr)
+		ctl.xcheck(err, "parsing sender domain")
+		hr.RecipientDomain, err = dns.ParseDomain(rcptdomstr)
+		ctl.xcheck(err, "parsing recipient domain")
+		hr, err = queue.HoldRuleAdd(ctx, log, hr)
+		ctl.xcheck(err, "add hold rule")
+		ctl.xwriteok()
+
+	case "queueholdrulesremove":
+		/* protocol:
+		> "queueholdrulesremove"
+		> id
+		< "ok" or error
+		*/
+		id, err := strconv.ParseInt(ctl.xread(), 10, 64)
+		ctl.xcheck(err, "parsing id")
+		err = queue.HoldRuleRemove(ctx, log, id)
+		ctl.xcheck(err, "remove hold rule")
+		ctl.xwriteok()
+
+	case "queuelist":
+		/* protocol:
+		> "queuelist"
+		> filters as json
+		> sort as json
+		< "ok"
+		< stream
+		*/
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		var s queue.Sort
+		xparseJSON(ctl, ctl.xread(), &s)
+		qmsgs, err := queue.List(ctx, f, s)
 		ctl.xcheck(err, "listing queue")
 		ctl.xwriteok()
 
 		xw := ctl.writer()
-		fmt.Fprintln(xw, "queue:")
+		fmt.Fprintln(xw, "messages:")
 		for _, qm := range qmsgs {
 			var lastAttempt string
 			if qm.LastAttempt != nil {
 				lastAttempt = time.Since(*qm.LastAttempt).Round(time.Second).String()
 			}
-			fmt.Fprintf(xw, "%5d %s from:%s to:%s next %s last %s error %q\n", qm.ID, qm.Queued.Format(time.RFC3339), qm.Sender().LogString(), qm.Recipient().LogString(), -time.Since(qm.NextAttempt).Round(time.Second), lastAttempt, qm.LastError)
+			fmt.Fprintf(xw, "%5d %s from:%s to:%s next %s last %s error %q\n", qm.ID, qm.Queued.Format(time.RFC3339), qm.Sender().LogString(), qm.Recipient().LogString(), -time.Since(qm.NextAttempt).Round(time.Second), lastAttempt, qm.LastResult().Error)
 		}
 		if len(qmsgs) == 0 {
-			fmt.Fprint(xw, "(empty)\n")
+			fmt.Fprint(xw, "(none)\n")
 		}
 		xw.xclose()
 
-	case "queuekick":
+	case "queueholdset":
 		/* protocol:
-		> "queuekick"
-		> id
-		> todomain
-		> recipient
-		> transport // if empty, transport is left unchanged; in future, we may want to differtiate between "leave unchanged" and "set to empty string".
-		< count
+		> "queueholdset"
+		> queuefilters as json
+		> "true" or "false"
 		< "ok" or error
+		< count
 		*/
 
-		idstr := ctl.xread()
-		todomain := ctl.xread()
-		recipient := ctl.xread()
-		transport := ctl.xread()
-		id, err := strconv.ParseInt(idstr, 10, 64)
-		if err != nil {
-			ctl.xwrite("0")
-			ctl.xcheck(err, "parsing id")
-		}
-
-		var xtransport *string
-		if transport != "" {
-			xtransport = &transport
-		}
-		count, err := queue.Kick(ctx, id, todomain, recipient, xtransport)
-		ctl.xcheck(err, "kicking queue")
-		ctl.xwrite(fmt.Sprintf("%d", count))
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		hold := ctl.xread() == "true"
+		count, err := queue.HoldSet(ctx, f, hold)
+		ctl.xcheck(err, "setting on hold status for messages")
 		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queueschedule":
+		/* protocol:
+		> "queueschedule"
+		> queuefilters as json
+		> relative to now
+		> duration
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		relnow := ctl.xread()
+		d, err := time.ParseDuration(ctl.xread())
+		ctl.xcheck(err, "parsing duration for next delivery attempt")
+		var count int
+		if relnow == "" {
+			count, err = queue.NextAttemptAdd(ctx, f, d)
+		} else {
+			count, err = queue.NextAttemptSet(ctx, f, time.Now().Add(d))
+		}
+		ctl.xcheck(err, "setting next delivery attempts in queue")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queuetransport":
+		/* protocol:
+		> "queuetransport"
+		> queuefilters as json
+		> transport
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		transport := ctl.xread()
+		count, err := queue.TransportSet(ctx, f, transport)
+		ctl.xcheck(err, "adding to next delivery attempts in queue")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queuerequiretls":
+		/* protocol:
+		> "queuerequiretls"
+		> queuefilters as json
+		> reqtls (empty string, "true" or "false")
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		reqtls := ctl.xread()
+		var req *bool
+		switch reqtls {
+		case "":
+		case "true":
+			v := true
+			req = &v
+		case "false":
+			v := false
+			req = &v
+		default:
+			ctl.xcheck(fmt.Errorf("unknown value %q", reqtls), "parsing value")
+		}
+		count, err := queue.RequireTLSSet(ctx, f, req)
+		ctl.xcheck(err, "setting tls requirements on messages in queue")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queuefail":
+		/* protocol:
+		> "queuefail"
+		> queuefilters as json
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		count, err := queue.Fail(ctx, log, f)
+		ctl.xcheck(err, "marking messages from queue as failed")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
 
 	case "queuedrop":
 		/* protocol:
 		> "queuedrop"
-		> id
-		> todomain
-		> recipient
-		< count
+		> queuefilters as json
 		< "ok" or error
+		< count
 		*/
 
-		idstr := ctl.xread()
-		todomain := ctl.xread()
-		recipient := ctl.xread()
-		id, err := strconv.ParseInt(idstr, 10, 64)
-		if err != nil {
-			ctl.xwrite("0")
-			ctl.xcheck(err, "parsing id")
-		}
-
-		count, err := queue.Drop(ctx, ctl.log, id, todomain, recipient)
+		var f queue.Filter
+		xparseJSON(ctl, ctl.xread(), &f)
+		count, err := queue.Drop(ctx, log, f)
 		ctl.xcheck(err, "dropping messages from queue")
-		ctl.xwrite(fmt.Sprintf("%d", count))
 		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
 
 	case "queuedump":
 		/* protocol:
@@ -470,6 +618,325 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		ctl.xwriteok()
 		ctl.xstreamfrom(mr)
 
+	case "queueretiredlist":
+		/* protocol:
+		> "queueretiredlist"
+		> filters as json
+		> sort as json
+		< "ok"
+		< stream
+		*/
+		var f queue.RetiredFilter
+		xparseJSON(ctl, ctl.xread(), &f)
+		var s queue.RetiredSort
+		xparseJSON(ctl, ctl.xread(), &s)
+		qmsgs, err := queue.RetiredList(ctx, f, s)
+		ctl.xcheck(err, "listing retired queue")
+		ctl.xwriteok()
+
+		xw := ctl.writer()
+		fmt.Fprintln(xw, "retired messages:")
+		for _, qm := range qmsgs {
+			var lastAttempt string
+			if qm.LastAttempt != nil {
+				lastAttempt = time.Since(*qm.LastAttempt).Round(time.Second).String()
+			}
+			result := "failure"
+			if qm.Success {
+				result = "success"
+			}
+			sender, err := qm.Sender()
+			xcheckf(err, "parsing sender")
+			fmt.Fprintf(xw, "%5d %s %s from:%s to:%s last %s error %q\n", qm.ID, qm.Queued.Format(time.RFC3339), result, sender.LogString(), qm.Recipient().LogString(), lastAttempt, qm.LastResult().Error)
+		}
+		if len(qmsgs) == 0 {
+			fmt.Fprint(xw, "(none)\n")
+		}
+		xw.xclose()
+
+	case "queueretiredprint":
+		/* protocol:
+		> "queueretiredprint"
+		> id
+		< "ok"
+		< stream
+		*/
+		idstr := ctl.xread()
+		id, err := strconv.ParseInt(idstr, 10, 64)
+		if err != nil {
+			ctl.xcheck(err, "parsing id")
+		}
+		l, err := queue.RetiredList(ctx, queue.RetiredFilter{IDs: []int64{id}}, queue.RetiredSort{})
+		ctl.xcheck(err, "getting retired messages")
+		if len(l) == 0 {
+			ctl.xcheck(errors.New("not found"), "getting retired message")
+		}
+		m := l[0]
+		ctl.xwriteok()
+		xw := ctl.writer()
+		enc := json.NewEncoder(xw)
+		enc.SetIndent("", "\t")
+		err = enc.Encode(m)
+		ctl.xcheck(err, "encode retired message")
+		xw.xclose()
+
+	case "queuehooklist":
+		/* protocol:
+		> "queuehooklist"
+		> filters as json
+		> sort as json
+		< "ok"
+		< stream
+		*/
+		var f queue.HookFilter
+		xparseJSON(ctl, ctl.xread(), &f)
+		var s queue.HookSort
+		xparseJSON(ctl, ctl.xread(), &s)
+		hooks, err := queue.HookList(ctx, f, s)
+		ctl.xcheck(err, "listing webhooks")
+		ctl.xwriteok()
+
+		xw := ctl.writer()
+		fmt.Fprintln(xw, "webhooks:")
+		for _, h := range hooks {
+			var lastAttempt string
+			if len(h.Results) > 0 {
+				lastAttempt = time.Since(h.LastResult().Start).Round(time.Second).String()
+			}
+			fmt.Fprintf(xw, "%5d %s account:%s next %s last %s error %q url %s\n", h.ID, h.Submitted.Format(time.RFC3339), h.Account, time.Until(h.NextAttempt).Round(time.Second), lastAttempt, h.LastResult().Error, h.URL)
+		}
+		if len(hooks) == 0 {
+			fmt.Fprint(xw, "(none)\n")
+		}
+		xw.xclose()
+
+	case "queuehookschedule":
+		/* protocol:
+		> "queuehookschedule"
+		> hookfilters as json
+		> relative to now
+		> duration
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.HookFilter
+		xparseJSON(ctl, ctl.xread(), &f)
+		relnow := ctl.xread()
+		d, err := time.ParseDuration(ctl.xread())
+		ctl.xcheck(err, "parsing duration for next delivery attempt")
+		var count int
+		if relnow == "" {
+			count, err = queue.HookNextAttemptAdd(ctx, f, d)
+		} else {
+			count, err = queue.HookNextAttemptSet(ctx, f, time.Now().Add(d))
+		}
+		ctl.xcheck(err, "setting next delivery attempts in queue")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queuehookcancel":
+		/* protocol:
+		> "queuehookcancel"
+		> hookfilters as json
+		< "ok" or error
+		< count
+		*/
+
+		var f queue.HookFilter
+		xparseJSON(ctl, ctl.xread(), &f)
+		count, err := queue.HookCancel(ctx, log, f)
+		ctl.xcheck(err, "canceling webhooks in queue")
+		ctl.xwriteok()
+		ctl.xwrite(fmt.Sprintf("%d", count))
+
+	case "queuehookprint":
+		/* protocol:
+		> "queuehookprint"
+		> id
+		< "ok"
+		< stream
+		*/
+		idstr := ctl.xread()
+		id, err := strconv.ParseInt(idstr, 10, 64)
+		if err != nil {
+			ctl.xcheck(err, "parsing id")
+		}
+		l, err := queue.HookList(ctx, queue.HookFilter{IDs: []int64{id}}, queue.HookSort{})
+		ctl.xcheck(err, "getting webhooks")
+		if len(l) == 0 {
+			ctl.xcheck(errors.New("not found"), "getting webhook")
+		}
+		h := l[0]
+		ctl.xwriteok()
+		xw := ctl.writer()
+		enc := json.NewEncoder(xw)
+		enc.SetIndent("", "\t")
+		err = enc.Encode(h)
+		ctl.xcheck(err, "encode webhook")
+		xw.xclose()
+
+	case "queuehookretiredlist":
+		/* protocol:
+		> "queuehookretiredlist"
+		> filters as json
+		> sort as json
+		< "ok"
+		< stream
+		*/
+		var f queue.HookRetiredFilter
+		xparseJSON(ctl, ctl.xread(), &f)
+		var s queue.HookRetiredSort
+		xparseJSON(ctl, ctl.xread(), &s)
+		l, err := queue.HookRetiredList(ctx, f, s)
+		ctl.xcheck(err, "listing retired webhooks")
+		ctl.xwriteok()
+
+		xw := ctl.writer()
+		fmt.Fprintln(xw, "retired webhooks:")
+		for _, h := range l {
+			var lastAttempt string
+			if len(h.Results) > 0 {
+				lastAttempt = time.Since(h.LastResult().Start).Round(time.Second).String()
+			}
+			result := "success"
+			if !h.Success {
+				result = "failure"
+			}
+			fmt.Fprintf(xw, "%5d %s %s account:%s last %s error %q url %s\n", h.ID, h.Submitted.Format(time.RFC3339), result, h.Account, lastAttempt, h.LastResult().Error, h.URL)
+		}
+		if len(l) == 0 {
+			fmt.Fprint(xw, "(none)\n")
+		}
+		xw.xclose()
+
+	case "queuehookretiredprint":
+		/* protocol:
+		> "queuehookretiredprint"
+		> id
+		< "ok"
+		< stream
+		*/
+		idstr := ctl.xread()
+		id, err := strconv.ParseInt(idstr, 10, 64)
+		if err != nil {
+			ctl.xcheck(err, "parsing id")
+		}
+		l, err := queue.HookRetiredList(ctx, queue.HookRetiredFilter{IDs: []int64{id}}, queue.HookRetiredSort{})
+		ctl.xcheck(err, "getting retired webhooks")
+		if len(l) == 0 {
+			ctl.xcheck(errors.New("not found"), "getting retired webhook")
+		}
+		h := l[0]
+		ctl.xwriteok()
+		xw := ctl.writer()
+		enc := json.NewEncoder(xw)
+		enc.SetIndent("", "\t")
+		err = enc.Encode(h)
+		ctl.xcheck(err, "encode retired webhook")
+		xw.xclose()
+
+	case "queuesuppresslist":
+		/* protocol:
+		> "queuesuppresslist"
+		> account (or empty)
+		< "ok" or error
+		< stream
+		*/
+
+		account := ctl.xread()
+		l, err := queue.SuppressionList(ctx, account)
+		ctl.xcheck(err, "listing suppressions")
+		ctl.xwriteok()
+		xw := ctl.writer()
+		fmt.Fprintln(xw, "suppressions (account, address, manual, time added, base adddress, reason):")
+		for _, sup := range l {
+			manual := "No"
+			if sup.Manual {
+				manual = "Yes"
+			}
+			fmt.Fprintf(xw, "%q\t%q\t%s\t%s\t%q\t%q\n", sup.Account, sup.OriginalAddress, manual, sup.Created.Round(time.Second), sup.BaseAddress, sup.Reason)
+		}
+		if len(l) == 0 {
+			fmt.Fprintln(xw, "(none)")
+		}
+		xw.xclose()
+
+	case "queuesuppressadd":
+		/* protocol:
+		> "queuesuppressadd"
+		> account
+		> address
+		< "ok" or error
+		*/
+
+		account := ctl.xread()
+		address := ctl.xread()
+		_, ok := mox.Conf.Account(account)
+		if !ok {
+			ctl.xcheck(errors.New("unknown account"), "looking up account")
+		}
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		sup := webapi.Suppression{
+			Account: account,
+			Manual:  true,
+			Reason:  "added through mox cli",
+		}
+		err = queue.SuppressionAdd(ctx, addr.Path(), &sup)
+		ctl.xcheck(err, "adding suppression")
+		ctl.xwriteok()
+
+	case "queuesuppressremove":
+		/* protocol:
+		> "queuesuppressremove"
+		> account
+		> address
+		< "ok" or error
+		*/
+
+		account := ctl.xread()
+		address := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		err = queue.SuppressionRemove(ctx, account, addr.Path())
+		ctl.xcheck(err, "removing suppression")
+		ctl.xwriteok()
+
+	case "queuesuppresslookup":
+		/* protocol:
+		> "queuesuppresslookup"
+		> account or empty
+		> address
+		< "ok" or error
+		< stream
+		*/
+
+		account := ctl.xread()
+		address := ctl.xread()
+		if account != "" {
+			_, ok := mox.Conf.Account(account)
+			if !ok {
+				ctl.xcheck(errors.New("unknown account"), "looking up account")
+			}
+		}
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		sup, err := queue.SuppressionLookup(ctx, account, addr.Path())
+		ctl.xcheck(err, "looking up suppression")
+		ctl.xwriteok()
+		xw := ctl.writer()
+		if sup == nil {
+			fmt.Fprintln(xw, "not present")
+		} else {
+			manual := "no"
+			if sup.Manual {
+				manual = "yes"
+			}
+			fmt.Fprintf(xw, "present\nadded: %s\nmanual: %s\nbase address: %s\nreason: %q\n", sup.Created.Round(time.Second), manual, sup.BaseAddress, sup.Reason)
+		}
+		xw.xclose()
+
 	case "importmaildir", "importmbox":
 		mbox := cmd == "importmbox"
 		importctl(ctx, ctl, mbox)
@@ -552,6 +1019,165 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		ctl.xcheck(err, "removing address")
 		ctl.xwriteok()
 
+	case "aliaslist":
+		/* protocol:
+		> "aliaslist"
+		> domain
+		< "ok" or error
+		< stream
+		*/
+		domain := ctl.xread()
+		d, err := dns.ParseDomain(domain)
+		ctl.xcheck(err, "parsing domain")
+		dc, ok := mox.Conf.Domain(d)
+		if !ok {
+			ctl.xcheck(errors.New("no such domain"), "listing aliases")
+		}
+		ctl.xwriteok()
+		w := ctl.writer()
+		for _, a := range dc.Aliases {
+			lp, err := smtp.ParseLocalpart(a.LocalpartStr)
+			ctl.xcheck(err, "parsing alias localpart")
+			fmt.Fprintln(w, smtp.NewAddress(lp, a.Domain).Pack(true))
+		}
+		w.xclose()
+
+	case "aliasprint":
+		/* protocol:
+		> "aliasprint"
+		> address
+		< "ok" or error
+		< stream
+		*/
+		address := ctl.xread()
+		_, alias, ok := mox.Conf.AccountDestination(address)
+		if !ok {
+			ctl.xcheck(errors.New("no such address"), "looking up alias")
+		} else if alias == nil {
+			ctl.xcheck(errors.New("address not an alias"), "looking up alias")
+		}
+		ctl.xwriteok()
+		w := ctl.writer()
+		fmt.Fprintf(w, "# postpublic %v\n", alias.PostPublic)
+		fmt.Fprintf(w, "# listmembers %v\n", alias.ListMembers)
+		fmt.Fprintf(w, "# allowmsgfrom %v\n", alias.AllowMsgFrom)
+		fmt.Fprintln(w, "# members:")
+		for _, a := range alias.Addresses {
+			fmt.Fprintln(w, a)
+		}
+		w.xclose()
+
+	case "aliasadd":
+		/* protocol:
+		> "aliasadd"
+		> address
+		> json alias
+		< "ok" or error
+		*/
+		address := ctl.xread()
+		line := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		var alias config.Alias
+		xparseJSON(ctl, line, &alias)
+		err = mox.AliasAdd(ctx, addr, alias)
+		ctl.xcheck(err, "adding alias")
+		ctl.xwriteok()
+
+	case "aliasupdate":
+		/* protocol:
+		> "aliasupdate"
+		> alias
+		> "true" or "false" for postpublic
+		> "true" or "false" for listmembers
+		> "true" or "false" for allowmsgfrom
+		< "ok" or error
+		*/
+		address := ctl.xread()
+		postpublic := ctl.xread()
+		listmembers := ctl.xread()
+		allowmsgfrom := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		err = mox.DomainSave(ctx, addr.Domain.Name(), func(d *config.Domain) error {
+			a, ok := d.Aliases[addr.Localpart.String()]
+			if !ok {
+				return fmt.Errorf("alias does not exist")
+			}
+
+			switch postpublic {
+			case "false":
+				a.PostPublic = false
+			case "true":
+				a.PostPublic = true
+			}
+			switch listmembers {
+			case "false":
+				a.ListMembers = false
+			case "true":
+				a.ListMembers = true
+			}
+			switch allowmsgfrom {
+			case "false":
+				a.AllowMsgFrom = false
+			case "true":
+				a.AllowMsgFrom = true
+			}
+
+			d.Aliases = maps.Clone(d.Aliases)
+			d.Aliases[addr.Localpart.String()] = a
+			return nil
+		})
+		ctl.xcheck(err, "saving alias")
+		ctl.xwriteok()
+
+	case "aliasrm":
+		/* protocol:
+		> "aliasrm"
+		> alias
+		< "ok" or error
+		*/
+		address := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		err = mox.AliasRemove(ctx, addr)
+		ctl.xcheck(err, "removing alias")
+		ctl.xwriteok()
+
+	case "aliasaddaddr":
+		/* protocol:
+		> "aliasaddaddr"
+		> alias
+		> addresses as json
+		< "ok" or error
+		*/
+		address := ctl.xread()
+		line := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		var addresses []string
+		xparseJSON(ctl, line, &addresses)
+		err = mox.AliasAddressesAdd(ctx, addr, addresses)
+		ctl.xcheck(err, "adding addresses to alias")
+		ctl.xwriteok()
+
+	case "aliasrmaddr":
+		/* protocol:
+		> "aliasrmaddr"
+		> alias
+		> addresses as json
+		< "ok" or error
+		*/
+		address := ctl.xread()
+		line := ctl.xread()
+		addr, err := smtp.ParseAddress(address)
+		ctl.xcheck(err, "parsing address")
+		var addresses []string
+		xparseJSON(ctl, line, &addresses)
+		err = mox.AliasAddressesRemove(ctx, addr, addresses)
+		ctl.xcheck(err, "removing addresses to alias")
+		ctl.xwriteok()
+
 	case "loglevels":
 		/* protocol:
 		> "loglevels"
@@ -587,13 +1213,13 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		pkg := ctl.xread()
 		levelstr := ctl.xread()
 		if levelstr == "" {
-			mox.Conf.LogLevelRemove(ctl.log, pkg)
+			mox.Conf.LogLevelRemove(log, pkg)
 		} else {
 			level, ok := mlog.Levels[levelstr]
 			if !ok {
 				ctl.xerror("bad level")
 			}
-			mox.Conf.LogLevelSet(ctl.log, pkg, level)
+			mox.Conf.LogLevelSet(log, pkg, level)
 		}
 		ctl.xwriteok()
 
@@ -604,7 +1230,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		< "ok" or error
 		*/
 		account := ctl.xread()
-		acc, err := store.OpenAccount(ctl.log, account)
+		acc, err := store.OpenAccount(log, account)
 		ctl.xcheck(err, "open account")
 		defer func() {
 			if acc != nil {
@@ -629,7 +1255,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 			log.Check(err, "removing old junkfilter bloom filter file", slog.String("path", bloomPath))
 
 			// Open junk filter, this creates new files.
-			jf, _, err := acc.OpenJunkFilter(ctx, ctl.log)
+			jf, _, err := acc.OpenJunkFilter(ctx, log)
 			ctl.xcheck(err, "open new junk filter")
 			defer func() {
 				if jf == nil {
@@ -645,14 +1271,14 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 			q.FilterEqual("Expunged", false)
 			err = q.ForEach(func(m store.Message) error {
 				total++
-				ok, err := acc.TrainMessage(ctx, ctl.log, jf, m)
+				ok, err := acc.TrainMessage(ctx, log, jf, m)
 				if ok {
 					trained++
 				}
 				return err
 			})
 			ctl.xcheck(err, "training messages")
-			ctl.log.Info("retrained messages", slog.Int("total", total), slog.Int("trained", trained))
+			log.Info("retrained messages", slog.Int("total", total), slog.Int("trained", trained))
 
 			// Close junk filter, marking success.
 			err = jf.Close()
@@ -669,7 +1295,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		< stream
 		*/
 		account := ctl.xread()
-		acc, err := store.OpenAccount(ctl.log, account)
+		acc, err := store.OpenAccount(log, account)
 		ctl.xcheck(err, "open account")
 		defer func() {
 			if acc != nil {
@@ -744,7 +1370,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		const batchSize = 10000
 
 		xfixmsgsize := func(accName string) {
-			acc, err := store.OpenAccount(ctl.log, accName)
+			acc, err := store.OpenAccount(log, accName)
 			ctl.xcheck(err, "open account")
 			defer func() {
 				err := acc.Close()
@@ -879,7 +1505,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		const batchSize = 100
 
 		xreparseAccount := func(accName string) {
-			acc, err := store.OpenAccount(ctl.log, accName)
+			acc, err := store.OpenAccount(log, accName)
 			ctl.xcheck(err, "open account")
 			defer func() {
 				err := acc.Close()
@@ -955,7 +1581,7 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 		w := ctl.writer()
 
 		xreassignThreads := func(accName string) {
-			acc, err := store.OpenAccount(ctl.log, accName)
+			acc, err := store.OpenAccount(log, accName)
 			ctl.xcheck(err, "open account")
 			defer func() {
 				err := acc.Close()
@@ -963,20 +1589,20 @@ func servectlcmd(ctx context.Context, ctl *ctl, shutdown func()) {
 			}()
 
 			// We don't want to step on an existing upgrade process.
-			err = acc.ThreadingWait(ctl.log)
+			err = acc.ThreadingWait(log)
 			ctl.xcheck(err, "waiting for threading upgrade to finish")
 			// todo: should we try to continue if the threading upgrade failed? only if there is a chance it will succeed this time...
 
 			// todo: reassigning isn't atomic (in a single transaction), ideally it would be (bstore would need to be able to handle large updates).
 			const batchSize = 50000
-			total, err := acc.ResetThreading(ctx, ctl.log, batchSize, true)
+			total, err := acc.ResetThreading(ctx, log, batchSize, true)
 			ctl.xcheck(err, "resetting threading fields")
 			_, err = fmt.Fprintf(w, "New thread base subject assigned to %d message(s), starting to reassign threads...\n", total)
 			ctl.xcheck(err, "write")
 
 			// Assign threads again. Ideally we would do this in a single transaction, but
 			// bstore/boltdb cannot handle so many pending changes, so we set a high batchsize.
-			err = acc.AssignThreads(ctx, ctl.log, nil, 0, 50000, w)
+			err = acc.AssignThreads(ctx, log, nil, 0, 50000, w)
 			ctl.xcheck(err, "reassign threads")
 
 			_, err = fmt.Fprintf(w, "Threads reassigned. You should invalidate messages stored at imap clients with the \"mox bumpuidvalidity account [mailbox]\" command.\n")
