@@ -14,34 +14,20 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"log/slog"
 	mathrand "math/rand"
 	"time"
 
-	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promauto"
-
 	"github.com/mjl-/mox/dkim"
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/mlog"
 	"github.com/mjl-/mox/publicsuffix"
 	"github.com/mjl-/mox/spf"
+	"github.com/mjl-/mox/stub"
 )
 
-var xlog = mlog.New("dmarc")
-
 var (
-	metricDMARCVerify = promauto.NewHistogramVec(
-		prometheus.HistogramOpts{
-			Name:    "mox_dmarc_verify_duration_seconds",
-			Help:    "DMARC verify, including lookup, duration and result.",
-			Buckets: []float64{0.001, 0.005, 0.01, 0.05, 0.100, 0.5, 1, 5, 10, 20},
-		},
-		[]string{
-			"status",
-			"reject", // yes/no
-			"use",    // yes/no, if policy is used after random selection
-		},
-	)
+	MetricVerify stub.HistogramVec = stub.HistogramVecIgnore{}
 )
 
 // link errata:
@@ -71,16 +57,21 @@ const (
 // Result is a DMARC policy evaluation.
 type Result struct {
 	// Whether to reject the message based on policies. If false, the message should
-	// not necessarily be accepted, e.g. due to reputation or content-based analysis.
+	// not necessarily be accepted: other checks such as reputation-based and
+	// content-based analysis may lead to reject the message.
 	Reject bool
 	// Result of DMARC validation. A message can fail validation, but still
 	// not be rejected, e.g. if the policy is "none".
-	Status Status
+	Status          Status
+	AlignedSPFPass  bool
+	AlignedDKIMPass bool
 	// Domain with the DMARC DNS record. May be the organizational domain instead of
 	// the domain in the From-header.
 	Domain dns.Domain
 	// Parsed DMARC record.
 	Record *Record
+	// Whether DMARC DNS response was DNSSEC-signed, regardless of whether SPF/DKIM records were DNSSEC-signed.
+	RecordAuthentic bool
 	// Details about possible error condition, e.g. when parsing the DMARC record failed.
 	Err error
 }
@@ -93,36 +84,45 @@ type Result struct {
 // domain is determined using the public suffix list. E.g. for
 // "sub.example.com", the organizational domain is "example.com". The returned
 // domain is the domain with the DMARC record.
-func Lookup(ctx context.Context, resolver dns.Resolver, from dns.Domain) (status Status, domain dns.Domain, record *Record, txt string, rerr error) {
-	log := xlog.WithContext(ctx)
+//
+// rauthentic indicates if the DNS results were DNSSEC-verified.
+func Lookup(ctx context.Context, elog *slog.Logger, resolver dns.Resolver, msgFrom dns.Domain) (status Status, domain dns.Domain, record *Record, txt string, rauthentic bool, rerr error) {
+	log := mlog.New("dmarc", elog)
 	start := time.Now()
 	defer func() {
-		log.Debugx("dmarc lookup result", rerr, mlog.Field("fromdomain", from), mlog.Field("status", status), mlog.Field("domain", domain), mlog.Field("record", record), mlog.Field("duration", time.Since(start)))
+		log.Debugx("dmarc lookup result", rerr,
+			slog.Any("fromdomain", msgFrom),
+			slog.Any("status", status),
+			slog.Any("domain", domain),
+			slog.Any("record", record),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
 	// ../rfc/7489:859 ../rfc/7489:1370
-	domain = from
-	status, record, txt, err := lookupRecord(ctx, resolver, domain)
+	domain = msgFrom
+	status, record, txt, authentic, err := lookupRecord(ctx, resolver, domain)
 	if status != StatusNone {
-		return status, domain, record, txt, err
+		return status, domain, record, txt, authentic, err
 	}
 	if record == nil {
 		// ../rfc/7489:761 ../rfc/7489:1377
-		domain = publicsuffix.Lookup(ctx, from)
-		if domain == from {
-			return StatusNone, domain, nil, txt, err
+		domain = publicsuffix.Lookup(ctx, log.Logger, msgFrom)
+		if domain == msgFrom {
+			return StatusNone, domain, nil, txt, authentic, err
 		}
 
-		status, record, txt, err = lookupRecord(ctx, resolver, domain)
+		var xauth bool
+		status, record, txt, xauth, err = lookupRecord(ctx, resolver, domain)
+		authentic = authentic && xauth
 	}
-	return status, domain, record, txt, err
+	return status, domain, record, txt, authentic, err
 }
 
-func lookupRecord(ctx context.Context, resolver dns.Resolver, domain dns.Domain) (Status, *Record, string, error) {
+func lookupRecord(ctx context.Context, resolver dns.Resolver, domain dns.Domain) (Status, *Record, string, bool, error) {
 	name := "_dmarc." + domain.ASCII + "."
-	txts, err := dns.WithPackage(resolver, "dmarc").LookupTXT(ctx, name)
+	txts, result, err := dns.WithPackage(resolver, "dmarc").LookupTXT(ctx, name)
 	if err != nil && !dns.IsNotFound(err) {
-		return StatusTemperror, nil, "", fmt.Errorf("%w: %s", ErrDNS, err)
+		return StatusTemperror, nil, "", result.Authentic, fmt.Errorf("%w: %s", ErrDNS, err)
 	}
 	var record *Record
 	var text string
@@ -133,27 +133,28 @@ func lookupRecord(ctx context.Context, resolver dns.Resolver, domain dns.Domain)
 			// ../rfc/7489:1374
 			continue
 		} else if err != nil {
-			return StatusPermerror, nil, text, fmt.Errorf("%w: %s", ErrSyntax, err)
+			return StatusPermerror, nil, text, result.Authentic, fmt.Errorf("%w: %s", ErrSyntax, err)
 		}
 		if record != nil {
-			// ../ ../rfc/7489:1388
-			return StatusNone, nil, "", ErrMultipleRecords
+			// ../rfc/7489:1388
+			return StatusNone, nil, "", result.Authentic, ErrMultipleRecords
 		}
 		text = txt
 		record = r
 		rerr = nil
 	}
-	return StatusNone, record, text, rerr
+	return StatusNone, record, text, result.Authentic, rerr
 }
 
-func lookupReportsRecord(ctx context.Context, resolver dns.Resolver, dmarcDomain, extDestDomain dns.Domain) (Status, *Record, string, error) {
+func lookupReportsRecord(ctx context.Context, resolver dns.Resolver, dmarcDomain, extDestDomain dns.Domain) (Status, []*Record, []string, bool, error) {
+	// ../rfc/7489:1566
 	name := dmarcDomain.ASCII + "._report._dmarc." + extDestDomain.ASCII + "."
-	txts, err := dns.WithPackage(resolver, "dmarc").LookupTXT(ctx, name)
+	txts, result, err := dns.WithPackage(resolver, "dmarc").LookupTXT(ctx, name)
 	if err != nil && !dns.IsNotFound(err) {
-		return StatusTemperror, nil, "", fmt.Errorf("%w: %s", ErrDNS, err)
+		return StatusTemperror, nil, nil, result.Authentic, fmt.Errorf("%w: %s", ErrDNS, err)
 	}
-	var record *Record
-	var text string
+	var records []*Record
+	var texts []string
 	var rerr error = ErrNoRecord
 	for _, txt := range txts {
 		r, isdmarc, err := ParseRecordNoRequired(txt)
@@ -165,42 +166,49 @@ func lookupReportsRecord(ctx context.Context, resolver dns.Resolver, dmarcDomain
 			r, isdmarc, err = &xr, true, nil
 		}
 		if !isdmarc {
-			// ../rfc/7489:1374
+			// ../rfc/7489:1586
 			continue
-		} else if err != nil {
-			return StatusPermerror, nil, text, fmt.Errorf("%w: %s", ErrSyntax, err)
 		}
-		if record != nil {
-			// ../ ../rfc/7489:1388
-			return StatusNone, nil, "", ErrMultipleRecords
+		texts = append(texts, txt)
+		records = append(records, r)
+		if err != nil {
+			return StatusPermerror, records, texts, result.Authentic, fmt.Errorf("%w: %s", ErrSyntax, err)
 		}
-		text = txt
-		record = r
+		// Multiple records are allowed for the _report record, unlike for policies. ../rfc/7489:1593
 		rerr = nil
 	}
-	return StatusNone, record, text, rerr
+	return StatusNone, records, texts, result.Authentic, rerr
 }
 
 // LookupExternalReportsAccepted returns whether the extDestDomain has opted in
 // to receiving dmarc reports for dmarcDomain (where the dmarc record was found),
 // through a "._report._dmarc." DNS TXT DMARC record.
 //
-// Callers should look at status for interpretation, not err, because err will
-// be set to ErrNoRecord when the DNS TXT record isn't present, which means the
-// extDestDomain does not opt in (not a failure condition).
+// accepts is true if the external domain has opted in.
+// If a temporary error occurred, the returned status is StatusTemperror, and a
+// later retry may give an authoritative result.
+// The returned error is ErrNoRecord if no opt-in DNS record exists, which is
+// not a failure condition.
 //
 // The normally invalid "v=DMARC1" record is accepted since it is used as
 // example in RFC 7489.
-func LookupExternalReportsAccepted(ctx context.Context, resolver dns.Resolver, dmarcDomain dns.Domain, extDestDomain dns.Domain) (accepts bool, status Status, record *Record, txt string, rerr error) {
-	log := xlog.WithContext(ctx)
+//
+// authentic indicates if the DNS results were DNSSEC-verified.
+func LookupExternalReportsAccepted(ctx context.Context, elog *slog.Logger, resolver dns.Resolver, dmarcDomain dns.Domain, extDestDomain dns.Domain) (accepts bool, status Status, records []*Record, txts []string, authentic bool, rerr error) {
+	log := mlog.New("dmarc", elog)
 	start := time.Now()
 	defer func() {
-		log.Debugx("dmarc externalreports result", rerr, mlog.Field("accepts", accepts), mlog.Field("dmarcdomain", dmarcDomain), mlog.Field("extdestdomain", extDestDomain), mlog.Field("record", record), mlog.Field("duration", time.Since(start)))
+		log.Debugx("dmarc externalreports result", rerr,
+			slog.Bool("accepts", accepts),
+			slog.Any("dmarcdomain", dmarcDomain),
+			slog.Any("extdestdomain", extDestDomain),
+			slog.Any("records", records),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
-	status, record, txt, rerr = lookupReportsRecord(ctx, resolver, dmarcDomain, extDestDomain)
+	status, records, txts, authentic, rerr = lookupReportsRecord(ctx, resolver, dmarcDomain, extDestDomain)
 	accepts = rerr == nil
-	return accepts, status, record, txt, rerr
+	return accepts, status, records, txts, authentic, rerr
 }
 
 // Verify evaluates the DMARC policy for the domain in the From-header of a
@@ -214,9 +222,10 @@ func LookupExternalReportsAccepted(ctx context.Context, resolver dns.Resolver, d
 // Verify always returns the result of verifying the DMARC policy
 // against the message (for inclusion in Authentication-Result headers).
 //
-// useResult indicates if the result should be applied in a policy decision.
-func Verify(ctx context.Context, resolver dns.Resolver, from dns.Domain, dkimResults []dkim.Result, spfResult spf.Status, spfIdentity *dns.Domain, applyRandomPercentage bool) (useResult bool, result Result) {
-	log := xlog.WithContext(ctx)
+// useResult indicates if the result should be applied in a policy decision,
+// based on the "pct" field in the DMARC record.
+func Verify(ctx context.Context, elog *slog.Logger, resolver dns.Resolver, msgFrom dns.Domain, dkimResults []dkim.Result, spfResult spf.Status, spfIdentity *dns.Domain, applyRandomPercentage bool) (useResult bool, result Result) {
+	log := mlog.New("dmarc", elog)
 	start := time.Now()
 	defer func() {
 		use := "no"
@@ -227,25 +236,33 @@ func Verify(ctx context.Context, resolver dns.Resolver, from dns.Domain, dkimRes
 		if result.Reject {
 			reject = "yes"
 		}
-		metricDMARCVerify.WithLabelValues(string(result.Status), reject, use).Observe(float64(time.Since(start)) / float64(time.Second))
-		log.Debugx("dmarc verify result", result.Err, mlog.Field("fromdomain", from), mlog.Field("dkimresults", dkimResults), mlog.Field("spfresult", spfResult), mlog.Field("status", result.Status), mlog.Field("reject", result.Reject), mlog.Field("use", useResult), mlog.Field("duration", time.Since(start)))
+		MetricVerify.ObserveLabels(float64(time.Since(start))/float64(time.Second), string(result.Status), reject, use)
+		log.Debugx("dmarc verify result", result.Err,
+			slog.Any("fromdomain", msgFrom),
+			slog.Any("dkimresults", dkimResults),
+			slog.Any("spfresult", spfResult),
+			slog.Any("status", result.Status),
+			slog.Bool("reject", result.Reject),
+			slog.Bool("use", useResult),
+			slog.Duration("duration", time.Since(start)))
 	}()
 
-	status, recordDomain, record, _, err := Lookup(ctx, resolver, from)
+	status, recordDomain, record, _, authentic, err := Lookup(ctx, log.Logger, resolver, msgFrom)
 	if record == nil {
-		return false, Result{false, status, recordDomain, record, err}
+		return false, Result{false, status, false, false, recordDomain, record, authentic, err}
 	}
 	result.Domain = recordDomain
 	result.Record = record
+	result.RecordAuthentic = authentic
 
 	// Record can request sampling of messages to apply policy.
 	// See ../rfc/7489:1432
 	useResult = !applyRandomPercentage || record.Percentage == 100 || mathrand.Intn(100) < record.Percentage
 
-	// We reject treat "quarantine" and "reject" the same. Thus, we also don't
-	// "downgrade" from reject to quarantine if this message was sampled out.
+	// We treat "quarantine" and "reject" the same. Thus, we also don't "downgrade"
+	// from reject to quarantine if this message was sampled out.
 	// ../rfc/7489:1446 ../rfc/7489:1024
-	if recordDomain != from && record.SubdomainPolicy != PolicyEmpty {
+	if recordDomain != msgFrom && record.SubdomainPolicy != PolicyEmpty {
 		result.Reject = record.SubdomainPolicy != PolicyNone
 	} else {
 		result.Reject = record.Policy != PolicyNone
@@ -265,17 +282,15 @@ func Verify(ctx context.Context, resolver dns.Resolver, from dns.Domain, dkimRes
 		if r, ok := pubsuffixes[name]; ok {
 			return r
 		}
-		r := publicsuffix.Lookup(ctx, name)
+		r := publicsuffix.Lookup(ctx, log.Logger, name)
 		pubsuffixes[name] = r
 		return r
 	}
 
 	// ../rfc/7489:1319
 	// ../rfc/7489:544
-	if spfResult == spf.StatusPass && spfIdentity != nil && (*spfIdentity == from || result.Record.ASPF == "r" && pubsuffix(from) == pubsuffix(*spfIdentity)) {
-		result.Reject = false
-		result.Status = StatusPass
-		return
+	if spfResult == spf.StatusPass && spfIdentity != nil && (*spfIdentity == msgFrom || result.Record.ASPF == "r" && pubsuffix(msgFrom) == pubsuffix(*spfIdentity)) {
+		result.AlignedSPFPass = true
 	}
 
 	for _, dkimResult := range dkimResults {
@@ -285,12 +300,16 @@ func Verify(ctx context.Context, resolver dns.Resolver, from dns.Domain, dkimRes
 			continue
 		}
 		// ../rfc/7489:511
-		if dkimResult.Status == dkim.StatusPass && dkimResult.Sig != nil && (dkimResult.Sig.Domain == from || result.Record.ADKIM == "r" && pubsuffix(from) == pubsuffix(dkimResult.Sig.Domain)) {
+		if dkimResult.Status == dkim.StatusPass && dkimResult.Sig != nil && (dkimResult.Sig.Domain == msgFrom || result.Record.ADKIM == "r" && pubsuffix(msgFrom) == pubsuffix(dkimResult.Sig.Domain)) {
 			// ../rfc/7489:535
-			result.Reject = false
-			result.Status = StatusPass
-			return
+			result.AlignedDKIMPass = true
+			break
 		}
 	}
+
+	if result.AlignedSPFPass || result.AlignedDKIMPass {
+		result.Reject = false
+		result.Status = StatusPass
+	}
 	return
 }
