@@ -842,7 +842,7 @@ Period: %s - %s UTC
 			continue
 		}
 
-		qm := queue.MakeMsg(from.Path(), rcpt.address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now())
+		qm := queue.MakeMsg(from.Path(), rcpt.address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now(), subject)
 		// Don't try as long as regular deliveries, and stop before we would send the
 		// delayed DSN. Though we also won't send that due to IsDMARCReport.
 		qm.MaxAttempts = 5
@@ -875,7 +875,15 @@ Period: %s - %s UTC
 }
 
 func composeAggregateReport(ctx context.Context, log mlog.Log, mf *os.File, fromAddr smtp.Address, recipients []message.NameAddress, subject, text, filename string, reportXMLGzipFile *os.File) (msgPrefix string, has8bit, smtputf8 bool, messageID string, rerr error) {
-	xc := message.NewComposer(mf, 100*1024*1024)
+	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
+	smtputf8 = fromAddr.Localpart.IsInternational()
+	for _, r := range recipients {
+		if smtputf8 {
+			smtputf8 = r.Address.Localpart.IsInternational()
+			break
+		}
+	}
+	xc := message.NewComposer(mf, 100*1024*1024, smtputf8)
 	defer func() {
 		x := recover()
 		if x == nil {
@@ -888,14 +896,6 @@ func composeAggregateReport(ctx context.Context, log mlog.Log, mf *os.File, from
 		panic(x)
 	}()
 
-	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
-	for _, a := range recipients {
-		if a.Address.Localpart.IsInternational() {
-			xc.SMTPUTF8 = true
-			break
-		}
-	}
-
 	xc.HeaderAddrs("From", []message.NameAddress{{Address: fromAddr}})
 	xc.HeaderAddrs("To", recipients)
 	xc.Subject(subject)
@@ -911,7 +911,7 @@ func composeAggregateReport(ctx context.Context, log mlog.Log, mf *os.File, from
 	xc.Line()
 
 	// Textual part, just mentioning this is a DMARC report.
-	textBody, ct, cte := xc.TextPart(text)
+	textBody, ct, cte := xc.TextPart("plain", text)
 	textHdr := textproto.MIMEHeader{}
 	textHdr.Set("Content-Type", ct)
 	textHdr.Set("Content-Transfer-Encoding", cte)
@@ -997,7 +997,7 @@ Submitting-URI: %s
 			continue
 		}
 
-		qm := queue.MakeMsg(fromAddr.Path(), rcpt.Address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now())
+		qm := queue.MakeMsg(fromAddr.Path(), rcpt.Address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now(), subject)
 		// Don't try as long as regular deliveries, and stop before we would send the
 		// delayed DSN. Though we also won't send that due to IsDMARCReport.
 		qm.MaxAttempts = 5
@@ -1015,7 +1015,15 @@ Submitting-URI: %s
 }
 
 func composeErrorReport(ctx context.Context, log mlog.Log, mf *os.File, fromAddr smtp.Address, recipients []message.NameAddress, subject, text string) (msgPrefix string, has8bit, smtputf8 bool, messageID string, rerr error) {
-	xc := message.NewComposer(mf, 100*1024*1024)
+	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
+	smtputf8 = fromAddr.Localpart.IsInternational()
+	for _, r := range recipients {
+		if smtputf8 {
+			smtputf8 = r.Address.Localpart.IsInternational()
+			break
+		}
+	}
+	xc := message.NewComposer(mf, 100*1024*1024, smtputf8)
 	defer func() {
 		x := recover()
 		if x == nil {
@@ -1028,14 +1036,6 @@ func composeErrorReport(ctx context.Context, log mlog.Log, mf *os.File, fromAddr
 		panic(x)
 	}()
 
-	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
-	for _, a := range recipients {
-		if a.Address.Localpart.IsInternational() {
-			xc.SMTPUTF8 = true
-			break
-		}
-	}
-
 	xc.HeaderAddrs("From", []message.NameAddress{{Address: fromAddr}})
 	xc.HeaderAddrs("To", recipients)
 	xc.Header("Subject", subject)
@@ -1045,7 +1045,7 @@ func composeErrorReport(ctx context.Context, log mlog.Log, mf *os.File, fromAddr
 	xc.Header("User-Agent", "mox/"+moxvar.Version)
 	xc.Header("MIME-Version", "1.0")
 
-	textBody, ct, cte := xc.TextPart(text)
+	textBody, ct, cte := xc.TextPart("plain", text)
 	xc.Header("Content-Type", ct)
 	xc.Header("Content-Transfer-Encoding", cte)
 	xc.Line()
