@@ -2,7 +2,6 @@ package dsn
 
 import (
 	"bytes"
-	"context"
 	"fmt"
 	"io"
 	"net"
@@ -11,15 +10,13 @@ import (
 	"testing"
 	"time"
 
-	"github.com/mjl-/mox/dkim"
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/message"
 	"github.com/mjl-/mox/mlog"
-	"github.com/mjl-/mox/mox-"
 	"github.com/mjl-/mox/smtp"
 )
 
-var xlog = mlog.New("dsn")
+var pkglog = mlog.New("dsn", nil)
 
 func xparseDomain(s string) dns.Domain {
 	d, err := dns.ParseDomain(s)
@@ -35,7 +32,7 @@ func xparseIPDomain(s string) dns.IPDomain {
 
 func tparseMessage(t *testing.T, data []byte, nparts int) (*Message, *message.Part) {
 	t.Helper()
-	m, p, err := Parse(xlog, bytes.NewReader(data))
+	m, p, err := Parse(pkglog.Logger, bytes.NewReader(data))
 	if err != nil {
 		t.Fatalf("parsing dsn: %v", err)
 	}
@@ -74,7 +71,7 @@ func tcompareReader(t *testing.T, r io.Reader, exp []byte) {
 }
 
 func TestDSN(t *testing.T) {
-	log := mlog.New("dsn")
+	log := mlog.New("dsn", nil)
 
 	now := time.Now()
 
@@ -82,14 +79,16 @@ func TestDSN(t *testing.T) {
 	m := Message{
 		SMTPUTF8: false,
 
-		From:     smtp.Path{Localpart: "postmaster", IPDomain: xparseIPDomain("mox.example")},
-		To:       smtp.Path{Localpart: "mjl", IPDomain: xparseIPDomain("remote.example")},
-		Subject:  "dsn",
-		TextBody: "delivery failure\n",
+		From:      smtp.Path{Localpart: "postmaster", IPDomain: xparseIPDomain("mox.example")},
+		To:        smtp.Path{Localpart: "mjl", IPDomain: xparseIPDomain("remote.example")},
+		Subject:   "dsn",
+		MessageID: "test@localhost",
+		TextBody:  "delivery failure\n",
 
-		ReportingMTA:    "mox.example",
-		ReceivedFromMTA: smtp.Ehlo{Name: xparseIPDomain("relay.example"), ConnIP: net.ParseIP("10.10.10.10")},
-		ArrivalDate:     now,
+		ReportingMTA:         "mox.example",
+		ReceivedFromMTA:      smtp.Ehlo{Name: xparseIPDomain("relay.example"), ConnIP: net.ParseIP("10.10.10.10")},
+		ArrivalDate:          now,
+		FutureReleaseRequest: "for;123",
 
 		Recipients: []Recipient{
 			{
@@ -106,6 +105,7 @@ func TestDSN(t *testing.T) {
 	if err != nil {
 		t.Fatalf("composing dsn: %v", err)
 	}
+
 	pmsg, part := tparseMessage(t, msgbuf, 3)
 	tcheckType(t, part, "multipart", "report", "")
 	tcheckType(t, &part.Parts[0], "text", "plain", "7bit")
@@ -129,35 +129,15 @@ func TestDSN(t *testing.T) {
 	tcompareReader(t, part.Parts[2].Reader(), m.Original)
 	tcompare(t, pmsg.Recipients[0].FinalRecipient, m.Recipients[0].FinalRecipient)
 
-	// Test for valid DKIM signature.
-	mox.Context = context.Background()
-	mox.ConfigStaticPath = "../testdata/dsn/mox.conf"
-	mox.MustLoadConfig(true, false)
-	msgbuf, err = m.Compose(log, false)
-	if err != nil {
-		t.Fatalf("composing utf-8 dsn with utf-8 support: %v", err)
-	}
-	resolver := &dns.MockResolver{
-		TXT: map[string][]string{
-			"testsel._domainkey.mox.example.": {"v=DKIM1;h=sha256;t=s;p=MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA3ZId3ys70VFspp/VMFaxMOrNjHNPg04NOE1iShih16b3Ex7hHBOgC1UvTGSmrMlbCB1OxTXkvf6jW6S4oYRnZYVNygH6zKUwYYhaSaGIg1xA/fDn+IgcTRyLoXizMUgUgpTGyxhNrwIIWv+i7jjbs3TKpP3NU4owQ/rxowmSNqg+fHIF1likSvXvljYS" + "jaFXXnWfYibW7TdDCFFpN4sB5o13+as0u4vLw6MvOi59B1tLype1LcHpi1b9PfxNtznTTdet3kL0paxIcWtKHT0LDPUos8YYmiPa5nGbUqlC7d+4YT2jQPvwGxCws1oo2Tw6nj1UaihneYGAyvEky49FBwIDAQAB"},
-		},
-	}
-	results, err := dkim.Verify(context.Background(), resolver, false, func(*dkim.Sig) error { return nil }, bytes.NewReader(msgbuf), false)
-	if err != nil {
-		t.Fatalf("dkim verify: %v", err)
-	}
-	if len(results) != 1 || results[0].Status != dkim.StatusPass {
-		t.Fatalf("dkim result not pass, %#v", results)
-	}
-
 	// An utf-8 message.
 	m = Message{
 		SMTPUTF8: true,
 
-		From:     smtp.Path{Localpart: "postmæster", IPDomain: xparseIPDomain("møx.example")},
-		To:       smtp.Path{Localpart: "møx", IPDomain: xparseIPDomain("remøte.example")},
-		Subject:  "dsn¡",
-		TextBody: "delivery failure¿\n",
+		From:      smtp.Path{Localpart: "postmæster", IPDomain: xparseIPDomain("møx.example")},
+		To:        smtp.Path{Localpart: "møx", IPDomain: xparseIPDomain("remøte.example")},
+		Subject:   "dsn¡",
+		MessageID: "test@localhost",
+		TextBody:  "delivery failure¿\n",
 
 		ReportingMTA:    "mox.example",
 		ReceivedFromMTA: smtp.Ehlo{Name: xparseIPDomain("reläy.example"), ConnIP: net.ParseIP("10.10.10.10")},
@@ -212,34 +192,3 @@ func TestDSN(t *testing.T) {
 	tcheckType(t, &part.Parts[1], "message", "global-delivery-status", "8bit")
 	tcompare(t, pmsg.Recipients[0].FinalRecipient, m.Recipients[0].FinalRecipient)
 }
-
-func TestCode(t *testing.T) {
-	testCodeLine := func(line, ecode, rest string) {
-		t.Helper()
-		e, r := codeLine(line)
-		if e != ecode || r != rest {
-			t.Fatalf("codeLine %q: got %q %q, expected %q %q", line, e, r, ecode, rest)
-		}
-	}
-	testCodeLine("4.0.0", "4.0.0", "")
-	testCodeLine("4.0.0 more", "4.0.0", "more")
-	testCodeLine("other", "", "other")
-	testCodeLine("other more", "", "other more")
-
-	testHasCode := func(line string, exp bool) {
-		t.Helper()
-		got := HasCode(line)
-		if got != exp {
-			t.Fatalf("HasCode %q: got %v, expected %v", line, got, exp)
-		}
-	}
-	testHasCode("4.0.0", true)
-	testHasCode("5.7.28", true)
-	testHasCode("10.0.0", false) // first number must be single digit.
-	testHasCode("4.1.1 more", true)
-	testHasCode("other ", false)
-	testHasCode("4.2.", false)
-	testHasCode("4.2. ", false)
-	testHasCode(" 4.2.4", false)
-	testHasCode(" 4.2.4 ", false)
-}
