@@ -35,6 +35,7 @@ import (
 	"github.com/mjl-/mox/ratelimit"
 	"github.com/mjl-/mox/webaccount"
 	"github.com/mjl-/mox/webadmin"
+	"github.com/mjl-/mox/webapisrv"
 	"github.com/mjl-/mox/webmail"
 )
 
@@ -577,6 +578,30 @@ func Listen() {
 		if maxMsgSize == 0 {
 			maxMsgSize = config.DefaultMaxMsgSize
 		}
+
+		if l.WebAPIHTTP.Enabled {
+			port := config.Port(l.WebAPIHTTP.Port, 80)
+			path := "/webapi/"
+			if l.WebAPIHTTP.Path != "" {
+				path = l.WebAPIHTTP.Path
+			}
+			srv := ensureServe(false, port, "webapi-http at "+path)
+			handler := safeHeaders(http.StripPrefix(path[:len(path)-1], webapisrv.NewServer(maxMsgSize, path, l.WebAPIHTTP.Forwarded)))
+			srv.Handle("webapi", nil, path, handler)
+			redirectToTrailingSlash(srv, "webapi", path)
+		}
+		if l.WebAPIHTTPS.Enabled {
+			port := config.Port(l.WebAPIHTTPS.Port, 443)
+			path := "/webapi/"
+			if l.WebAPIHTTPS.Path != "" {
+				path = l.WebAPIHTTPS.Path
+			}
+			srv := ensureServe(true, port, "webapi-https at "+path)
+			handler := safeHeaders(http.StripPrefix(path[:len(path)-1], webapisrv.NewServer(maxMsgSize, path, l.WebAPIHTTPS.Forwarded)))
+			srv.Handle("webapi", nil, path, handler)
+			redirectToTrailingSlash(srv, "webapi", path)
+		}
+
 		if l.WebmailHTTP.Enabled {
 			port := config.Port(l.WebmailHTTP.Port, 80)
 			path := "/webmail/"
@@ -584,7 +609,14 @@ func Listen() {
 				path = l.WebmailHTTP.Path
 			}
 			srv := ensureServe(false, port, "webmail-http at "+path)
-			handler := http.StripPrefix(path[:len(path)-1], http.HandlerFunc(webmail.Handler(maxMsgSize, path, l.WebmailHTTP.Forwarded)))
+			var accountPath string
+			if l.AccountHTTP.Enabled {
+				accountPath = "/"
+				if l.AccountHTTP.Path != "" {
+					accountPath = l.AccountHTTP.Path
+				}
+			}
+			handler := http.StripPrefix(path[:len(path)-1], http.HandlerFunc(webmail.Handler(maxMsgSize, path, l.WebmailHTTP.Forwarded, accountPath)))
 			srv.Handle("webmail", nil, path, handler)
 			redirectToTrailingSlash(srv, "webmail", path)
 		}
@@ -595,7 +627,14 @@ func Listen() {
 				path = l.WebmailHTTPS.Path
 			}
 			srv := ensureServe(true, port, "webmail-https at "+path)
-			handler := http.StripPrefix(path[:len(path)-1], http.HandlerFunc(webmail.Handler(maxMsgSize, path, l.WebmailHTTPS.Forwarded)))
+			var accountPath string
+			if l.AccountHTTPS.Enabled {
+				accountPath = "/"
+				if l.AccountHTTPS.Path != "" {
+					accountPath = l.AccountHTTPS.Path
+				}
+			}
+			handler := http.StripPrefix(path[:len(path)-1], http.HandlerFunc(webmail.Handler(maxMsgSize, path, l.WebmailHTTPS.Forwarded, accountPath)))
 			srv.Handle("webmail", nil, path, handler)
 			redirectToTrailingSlash(srv, "webmail", path)
 		}
@@ -772,8 +811,9 @@ func listen1(ip string, port int, tlsConfig *tls.Config, name string, kinds []st
 	}
 
 	server := &http.Server{
-		Handler:           handler,
-		TLSConfig:         tlsConfig,
+		Handler: handler,
+		// Clone because our multiple Server.Serve calls modify config concurrently leading to data race.
+		TLSConfig:         tlsConfig.Clone(),
 		ReadHeaderTimeout: 30 * time.Second,
 		IdleTimeout:       65 * time.Second, // Chrome closes connections after 60 seconds, firefox after 115 seconds.
 		ErrorLog:          golog.New(mlog.LogWriter(pkglog.With(slog.String("pkg", "net/http")), slog.LevelInfo, protocol+" error"), "", 0),
