@@ -61,6 +61,8 @@ var (
 	Conf              = Config{Log: map[string]slog.Level{"": slog.LevelError}}
 )
 
+var ErrConfig = errors.New("config error")
+
 // Config as used in the code, a processed version of what is in the config file.
 //
 // Use methods to lookup a domain/account/address in the dynamic configuration.
@@ -78,6 +80,8 @@ type Config struct {
 	// case-insensitive, stripped of catchall separator) to account and address.
 	// Domains are IDNA names in utf8.
 	accountDestinations map[string]AccountDestination
+	// Like accountDestinations, but for aliases.
+	aliases map[string]config.Alias
 }
 
 type AccountDestination struct {
@@ -150,17 +154,26 @@ func (c *Config) withDynamicLock(fn func()) {
 
 // must be called with dynamic lock held.
 func (c *Config) loadDynamic() []error {
-	d, mtime, accDests, err := ParseDynamicConfig(context.Background(), pkglog, ConfigDynamicPath, c.Static)
+	d, mtime, accDests, aliases, err := ParseDynamicConfig(context.Background(), pkglog, ConfigDynamicPath, c.Static)
 	if err != nil {
 		return err
 	}
 	c.Dynamic = d
 	c.dynamicMtime = mtime
 	c.accountDestinations = accDests
+	c.aliases = aliases
 	c.allowACMEHosts(pkglog, true)
 	return nil
 }
 
+// DynamicConfig returns a shallow copy of the dynamic config. Must not be modified.
+func (c *Config) DynamicConfig() (config config.Dynamic) {
+	c.withDynamicLock(func() {
+		config = c.Dynamic // Shallow copy.
+	})
+	return
+}
+
 func (c *Config) Domains() (l []string) {
 	c.withDynamicLock(func() {
 		for name := range c.Dynamic.Domains {
@@ -183,10 +196,12 @@ func (c *Config) Accounts() (l []string) {
 }
 
 // DomainLocalparts returns a mapping of encoded localparts to account names for a
-// domain. An empty localpart is a catchall destination for a domain.
-func (c *Config) DomainLocalparts(d dns.Domain) map[string]string {
+// domain, and encoded localparts to aliases. An empty localpart is a catchall
+// destination for a domain.
+func (c *Config) DomainLocalparts(d dns.Domain) (map[string]string, map[string]config.Alias) {
 	suffix := "@" + d.Name()
 	m := map[string]string{}
+	aliases := map[string]config.Alias{}
 	c.withDynamicLock(func() {
 		for addr, ad := range c.accountDestinations {
 			if strings.HasSuffix(addr, suffix) {
@@ -197,8 +212,13 @@ func (c *Config) DomainLocalparts(d dns.Domain) map[string]string {
 				}
 			}
 		}
+		for addr, a := range c.aliases {
+			if strings.HasSuffix(addr, suffix) {
+				aliases[a.LocalpartStr] = a
+			}
+		}
 	})
-	return m
+	return m, aliases
 }
 
 func (c *Config) Domain(d dns.Domain) (dom config.Domain, ok bool) {
@@ -215,21 +235,20 @@ func (c *Config) Account(name string) (acc config.Account, ok bool) {
 	return
 }
 
-func (c *Config) AccountDestination(addr string) (accDests AccountDestination, ok bool) {
+func (c *Config) AccountDestination(addr string) (accDest AccountDestination, alias *config.Alias, ok bool) {
 	c.withDynamicLock(func() {
-		accDests, ok = c.accountDestinations[addr]
+		accDest, ok = c.accountDestinations[addr]
+		if !ok {
+			var a config.Alias
+			a, ok = c.aliases[addr]
+			if ok {
+				alias = &a
+			}
+		}
 	})
 	return
 }
 
-func (c *Config) WebServer() (r map[dns.Domain]dns.Domain, l []config.WebHandler) {
-	c.withDynamicLock(func() {
-		r = c.Dynamic.WebDNSDomainRedirects
-		l = c.Dynamic.WebHandlers
-	})
-	return r, l
-}
-
 func (c *Config) Routes(accountName string, domain dns.Domain) (accountRoutes, domainRoutes, globalRoutes []config.Route) {
 	c.withDynamicLock(func() {
 		acc := c.Dynamic.Accounts[accountName]
@@ -243,13 +262,6 @@ func (c *Config) Routes(accountName string, domain dns.Domain) (accountRoutes, d
 	return
 }
 
-func (c *Config) MonitorDNSBLs() (zones []dns.Domain) {
-	c.withDynamicLock(func() {
-		zones = c.Dynamic.MonitorDNSBLZones
-	})
-	return
-}
-
 func (c *Config) allowACMEHosts(log mlog.Log, checkACMEHosts bool) {
 	for _, l := range c.Static.Listeners {
 		if l.TLS == nil || l.TLS.ACME == "" {
@@ -317,10 +329,11 @@ func (c *Config) allowACMEHosts(log mlog.Log, checkACMEHosts bool) {
 // todo future: write config parsing & writing code that can read a config and remembers the exact tokens including newlines and comments, and can write back a modified file. the goal is to be able to write a config file automatically (after changing fields through the ui), but not loose comments and whitespace, to still get useful diffs for storing the config in a version control system.
 
 // must be called with lock held.
+// Returns ErrConfig if the configuration is not valid.
 func writeDynamic(ctx context.Context, log mlog.Log, c config.Dynamic) error {
-	accDests, errs := prepareDynamicConfig(ctx, log, ConfigDynamicPath, Conf.Static, &c)
+	accDests, aliases, errs := prepareDynamicConfig(ctx, log, ConfigDynamicPath, Conf.Static, &c)
 	if len(errs) > 0 {
-		return errs[0]
+		return fmt.Errorf("%w: %v", ErrConfig, errs[0])
 	}
 
 	var b bytes.Buffer
@@ -366,6 +379,7 @@ func writeDynamic(ctx context.Context, log mlog.Log, c config.Dynamic) error {
 	Conf.DynamicLastCheck = time.Now()
 	Conf.Dynamic = c
 	Conf.accountDestinations = accDests
+	Conf.aliases = aliases
 
 	Conf.allowACMEHosts(log, true)
 
@@ -405,7 +419,7 @@ func LoadConfig(ctx context.Context, log mlog.Log, doLoadTLSKeyCerts, checkACMEH
 // SetConfig sets a new config. Not to be used during normal operation.
 func SetConfig(c *Config) {
 	// Cannot just assign *c to Conf, it would copy the mutex.
-	Conf = Config{c.Static, sync.Mutex{}, c.Log, sync.Mutex{}, c.Dynamic, c.dynamicMtime, c.DynamicLastCheck, c.accountDestinations}
+	Conf = Config{c.Static, sync.Mutex{}, c.Log, sync.Mutex{}, c.Dynamic, c.dynamicMtime, c.DynamicLastCheck, c.accountDestinations, c.aliases}
 
 	// If we have non-standard CA roots, use them for all HTTPS requests.
 	if Conf.Static.TLS.CertPool != nil {
@@ -456,7 +470,7 @@ func ParseConfig(ctx context.Context, log mlog.Log, p string, checkOnly, doLoadT
 	}
 
 	pp := filepath.Join(filepath.Dir(p), "domains.conf")
-	c.Dynamic, c.dynamicMtime, c.accountDestinations, errs = ParseDynamicConfig(ctx, log, pp, c.Static)
+	c.Dynamic, c.dynamicMtime, c.accountDestinations, c.aliases, errs = ParseDynamicConfig(ctx, log, pp, c.Static)
 
 	if !checkOnly {
 		c.allowACMEHosts(log, checkACMEHosts)
@@ -929,6 +943,19 @@ func PrepareStaticConfig(ctx context.Context, log mlog.Log, configFile string, c
 		}
 	}
 
+	checkTransportDirect := func(name string, t *config.TransportDirect) {
+		if t.DisableIPv4 && t.DisableIPv6 {
+			addErrorf("transport %s: both IPv4 and IPv6 are disabled, enable at least one", name)
+		}
+		t.IPFamily = "ip"
+		if t.DisableIPv4 {
+			t.IPFamily = "ip6"
+		}
+		if t.DisableIPv6 {
+			t.IPFamily = "ip4"
+		}
+	}
+
 	for name, t := range c.Transports {
 		n := 0
 		if t.Submissions != nil {
@@ -947,6 +974,10 @@ func PrepareStaticConfig(ctx context.Context, log mlog.Log, configFile string, c
 			n++
 			checkTransportSocks(name, t.Socks)
 		}
+		if t.Direct != nil {
+			n++
+			checkTransportDirect(name, t.Direct)
+		}
 		if n > 1 {
 			addErrorf("transport %s: cannot have multiple methods in a transport", name)
 		}
@@ -979,7 +1010,7 @@ func PrepareStaticConfig(ctx context.Context, log mlog.Log, configFile string, c
 }
 
 // PrepareDynamicConfig parses the dynamic config file given a static file.
-func ParseDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string, static config.Static) (c config.Dynamic, mtime time.Time, accDests map[string]AccountDestination, errs []error) {
+func ParseDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string, static config.Static) (c config.Dynamic, mtime time.Time, accDests map[string]AccountDestination, aliases map[string]config.Alias, errs []error) {
 	addErrorf := func(format string, args ...any) {
 		errs = append(errs, fmt.Errorf(format, args...))
 	}
@@ -999,11 +1030,11 @@ func ParseDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string, s
 		return
 	}
 
-	accDests, errs = prepareDynamicConfig(ctx, log, dynamicPath, static, &c)
-	return c, fi.ModTime(), accDests, errs
+	accDests, aliases, errs = prepareDynamicConfig(ctx, log, dynamicPath, static, &c)
+	return c, fi.ModTime(), accDests, aliases, errs
 }
 
-func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string, static config.Static, c *config.Dynamic) (accDests map[string]AccountDestination, errs []error) {
+func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string, static config.Static, c *config.Dynamic) (accDests map[string]AccountDestination, aliases map[string]config.Alias, errs []error) {
 	addErrorf := func(format string, args ...any) {
 		errs = append(errs, fmt.Errorf(format, args...))
 	}
@@ -1024,6 +1055,7 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 	checkMailboxNormf(static.Postmaster.Mailbox, "postmaster mailbox")
 
 	accDests = map[string]AccountDestination{}
+	aliases = map[string]config.Alias{}
 
 	// Validate host TLSRPT account/address.
 	if static.HostTLSRPT.Account != "" {
@@ -1163,11 +1195,13 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 					addErrorf("rsa keys should be >= 1024 bits")
 				}
 				sel.Key = k
+				sel.Algorithm = fmt.Sprintf("rsa-%d", k.N.BitLen())
 			case ed25519.PrivateKey:
 				if sel.HashEffective != "sha256" {
 					addErrorf("hash algorithm %q is not supported with ed25519, only sha256 is", sel.HashEffective)
 				}
 				sel.Key = k
+				sel.Algorithm = "ed25519"
 			default:
 				addErrorf("private key type %T not yet supported, at selector %s in domain %s", key, name, d)
 			}
@@ -1255,8 +1289,55 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 			}
 			acc.NotJunkMailbox = r
 		}
+
+		acc.ParsedFromIDLoginAddresses = make([]smtp.Address, len(acc.FromIDLoginAddresses))
+		for i, s := range acc.FromIDLoginAddresses {
+			a, err := smtp.ParseAddress(s)
+			if err != nil {
+				addErrorf("invalid fromid login address %q in account %q: %v", s, accName, err)
+			}
+			// We check later on if address belongs to account.
+			dom, ok := c.Domains[a.Domain.Name()]
+			if !ok {
+				addErrorf("unknown domain in fromid login address %q for account %q", s, accName)
+			} else if dom.LocalpartCatchallSeparator == "" {
+				addErrorf("localpart catchall separator not configured for domain for fromid login address %q for account %q", s, accName)
+			}
+			acc.ParsedFromIDLoginAddresses[i] = a
+		}
+
+		// Clear any previously derived state.
+		acc.Aliases = nil
+
 		c.Accounts[accName] = acc
 
+		if acc.OutgoingWebhook != nil {
+			u, err := url.Parse(acc.OutgoingWebhook.URL)
+			if err == nil && (u.Scheme != "http" && u.Scheme != "https") {
+				err = errors.New("scheme must be http or https")
+			}
+			if err != nil {
+				addErrorf("parsing outgoing hook url %q in account %q: %v", acc.OutgoingWebhook.URL, accName, err)
+			}
+
+			// note: outgoing hook events are in ../queue/hooks.go, ../mox-/config.go, ../queue.go and ../webapi/gendoc.sh. keep in sync.
+			outgoingHookEvents := []string{"delivered", "suppressed", "delayed", "failed", "relayed", "expanded", "canceled", "unrecognized"}
+			for _, e := range acc.OutgoingWebhook.Events {
+				if !slices.Contains(outgoingHookEvents, e) {
+					addErrorf("unknown outgoing hook event %q", e)
+				}
+			}
+		}
+		if acc.IncomingWebhook != nil {
+			u, err := url.Parse(acc.IncomingWebhook.URL)
+			if err == nil && (u.Scheme != "http" && u.Scheme != "https") {
+				err = errors.New("scheme must be http or https")
+			}
+			if err != nil {
+				addErrorf("parsing incoming hook url %q in account %q: %v", acc.IncomingWebhook.URL, accName, err)
+			}
+		}
+
 		// todo deprecated: only localpart as keys for Destinations, we are replacing them with full addresses. if domains.conf is written, we won't have to do this again.
 		replaceLocalparts := map[string]string{}
 
@@ -1276,6 +1357,14 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 					}
 					c.Accounts[accName].Destinations[addrName].Rulesets[i].SMTPMailFromRegexpCompiled = r
 				}
+				if rs.MsgFromRegexp != "" {
+					n++
+					r, err := regexp.Compile(rs.MsgFromRegexp)
+					if err != nil {
+						addErrorf("invalid MsgFrom regular expression: %v", err)
+					}
+					c.Accounts[accName].Destinations[addrName].Rulesets[i].MsgFromRegexpCompiled = r
+				}
 				if rs.VerifiedDomain != "" {
 					n++
 					d, err := dns.ParseDomain(rs.VerifiedDomain)
@@ -1378,9 +1467,8 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 			origLP := address.Localpart
 			dc := c.Domains[address.Domain.Name()]
 			domainHasAddress[address.Domain.Name()] = true
-			if lp, err := CanonicalLocalpart(address.Localpart, dc); err != nil {
-				addErrorf("canonicalizing localpart %s: %v", address.Localpart, err)
-			} else if dc.LocalpartCatchallSeparator != "" && strings.Contains(string(address.Localpart), dc.LocalpartCatchallSeparator) {
+			lp := CanonicalLocalpart(address.Localpart, dc)
+			if dc.LocalpartCatchallSeparator != "" && strings.Contains(string(address.Localpart), dc.LocalpartCatchallSeparator) {
 				addErrorf("localpart of address %s includes domain catchall separator %s", address, dc.LocalpartCatchallSeparator)
 			} else {
 				address.Localpart = lp
@@ -1406,6 +1494,20 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 			}
 		}
 
+		// Now that all addresses are parsed, check if all fromid login addresses match
+		// configured addresses.
+		for i, a := range acc.ParsedFromIDLoginAddresses {
+			// For domain catchall.
+			if _, ok := accDests["@"+a.Domain.Name()]; ok {
+				continue
+			}
+			dc := c.Domains[a.Domain.Name()]
+			a.Localpart = CanonicalLocalpart(a.Localpart, dc)
+			if _, ok := accDests[a.Pack(true)]; !ok {
+				addErrorf("fromid login address %q for account %q does not match its destination addresses", acc.FromIDLoginAddresses[i], accName)
+			}
+		}
+
 		checkRoutes("routes for account", acc.Routes)
 	}
 
@@ -1501,6 +1603,86 @@ func prepareDynamicConfig(ctx context.Context, log mlog.Log, dynamicPath string,
 		c.Domains[d] = domain
 	}
 
+	// Aliases, per domain. Also add references to accounts.
+	for d, domain := range c.Domains {
+		for lpstr, a := range domain.Aliases {
+			var err error
+			a.LocalpartStr = lpstr
+			var clp smtp.Localpart
+			lp, err := smtp.ParseLocalpart(lpstr)
+			if err != nil {
+				addErrorf("domain %q: parsing localpart %q for alias: %v", d, lpstr, err)
+				continue
+			} else if domain.LocalpartCatchallSeparator != "" && strings.Contains(string(lp), domain.LocalpartCatchallSeparator) {
+				addErrorf("domain %q: alias %q contains localpart catchall separator", d, a.LocalpartStr)
+				continue
+			} else {
+				clp = CanonicalLocalpart(lp, domain)
+			}
+
+			addr := smtp.NewAddress(clp, domain.Domain).Pack(true)
+			if _, ok := aliases[addr]; ok {
+				addErrorf("domain %q: duplicate alias address %q", d, addr)
+				continue
+			}
+			if _, ok := accDests[addr]; ok {
+				addErrorf("domain %q: alias %q already present as regular address", d, addr)
+				continue
+			}
+			if len(a.Addresses) == 0 {
+				// Not currently possible, Addresses isn't optional.
+				addErrorf("domain %q: alias %q needs at least one destination address", d, addr)
+				continue
+			}
+			a.ParsedAddresses = make([]config.AliasAddress, 0, len(a.Addresses))
+			seen := map[string]bool{}
+			for _, destAddr := range a.Addresses {
+				da, err := smtp.ParseAddress(destAddr)
+				if err != nil {
+					addErrorf("domain %q: parsing destination address %q in alias %q: %v", d, destAddr, addr, err)
+					continue
+				}
+				dastr := da.Pack(true)
+				accDest, ok := accDests[dastr]
+				if !ok {
+					addErrorf("domain %q: alias %q references non-existent address %q", d, addr, destAddr)
+					continue
+				}
+				if seen[dastr] {
+					addErrorf("domain %q: alias %q has duplicate address %q", d, addr, destAddr)
+					continue
+				}
+				seen[dastr] = true
+				aa := config.AliasAddress{Address: da, AccountName: accDest.Account, Destination: accDest.Destination}
+				a.ParsedAddresses = append(a.ParsedAddresses, aa)
+			}
+			a.Domain = domain.Domain
+			c.Domains[d].Aliases[lpstr] = a
+			aliases[addr] = a
+
+			for _, aa := range a.ParsedAddresses {
+				acc := c.Accounts[aa.AccountName]
+				var addrs []string
+				if a.ListMembers {
+					addrs = make([]string, len(a.ParsedAddresses))
+					for i := range a.ParsedAddresses {
+						addrs[i] = a.ParsedAddresses[i].Address.Pack(true)
+					}
+				}
+				// Keep the non-sensitive fields.
+				accAlias := config.Alias{
+					PostPublic:   a.PostPublic,
+					ListMembers:  a.ListMembers,
+					AllowMsgFrom: a.AllowMsgFrom,
+					LocalpartStr: a.LocalpartStr,
+					Domain:       a.Domain,
+				}
+				acc.Aliases = append(acc.Aliases, config.AddressAlias{SubscriptionAddress: aa.Address.Pack(true), Alias: accAlias, MemberAddresses: addrs})
+				c.Accounts[aa.AccountName] = acc
+			}
+		}
+	}
+
 	// Check webserver configs.
 	if (len(c.WebDomainRedirects) > 0 || len(c.WebHandlers) > 0) && !haveWebserverListener {
 		addErrorf("WebDomainRedirects or WebHandlers configured but no listener with WebserverHTTP or WebserverHTTPS enabled")
