@@ -19,6 +19,7 @@ import (
 	"github.com/mjl-/adns"
 	"github.com/mjl-/bstore"
 
+	"github.com/mjl-/mox/config"
 	"github.com/mjl-/mox/dns"
 	"github.com/mjl-/mox/dsn"
 	"github.com/mjl-/mox/mlog"
@@ -29,6 +30,7 @@ import (
 	"github.com/mjl-/mox/smtpclient"
 	"github.com/mjl-/mox/store"
 	"github.com/mjl-/mox/tlsrpt"
+	"github.com/mjl-/mox/webhook"
 )
 
 // Increased each time an outgoing connection is made for direct delivery. Used by
@@ -110,7 +112,7 @@ type msgResp struct {
 // domain (MTA-STS), its policy type can be empty, in which case there is no
 // information (e.g. internal failure). hostResults are per-host details (DANE, one
 // per MX target).
-func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer, ourHostname dns.Domain, transportName string, msgs []*Msg, backoff time.Duration) (recipientDomainResult tlsrpt.Result, hostResults []tlsrpt.Result) {
+func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer, ourHostname dns.Domain, transportName string, transportDirect *config.TransportDirect, msgs []*Msg, backoff time.Duration) (recipientDomainResult tlsrpt.Result, hostResults []tlsrpt.Result) {
 	// High-level approach:
 	// - Resolve domain to deliver to (CNAME), and determine hosts to try to deliver to (MX)
 	// - Get MTA-STS policy for domain (optional). If present, only deliver to its
@@ -154,7 +156,7 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 		if permanent {
 			err = smtpclient.Error{Permanent: true, Err: err}
 		}
-		fail(ctx, qlog, msgs, m0.DialedIPs, backoff, dsn.NameIP{}, err)
+		failMsgsDB(qlog, msgs, m0.DialedIPs, backoff, dsn.NameIP{}, err)
 		return
 	}
 
@@ -174,7 +176,7 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 			} else {
 				qlog.Infox("mtasts lookup temporary error, aborting delivery attempt", err, slog.Any("domain", origNextHop))
 				recipientDomainResult.Summary.TotalFailureSessionCount++
-				fail(ctx, qlog, msgs, m0.DialedIPs, backoff, dsn.NameIP{}, err)
+				failMsgsDB(qlog, msgs, m0.DialedIPs, backoff, dsn.NameIP{}, err)
 				return
 			}
 		}
@@ -252,7 +254,7 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 			msgResps[i] = &msgResp{msg: msgs[i]}
 		}
 
-		result := deliverHost(nqlog, resolver, dialer, ourHostname, transportName, h, enforceMTASTS, haveMX, origNextHopAuthentic, origNextHop, expandedNextHopAuthentic, expandedNextHop, msgResps, tlsMode, tlsPKIX, &recipientDomainResult)
+		result := deliverHost(nqlog, resolver, dialer, ourHostname, transportName, transportDirect, h, enforceMTASTS, haveMX, origNextHopAuthentic, origNextHop, expandedNextHopAuthentic, expandedNextHop, msgResps, tlsMode, tlsPKIX, &recipientDomainResult)
 
 		var zerotype tlsrpt.PolicyType
 		if result.hostResult.Policy.Type != zerotype {
@@ -279,7 +281,7 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 				slog.Bool("enforcemtasts", enforceMTASTS),
 				slog.Bool("tlsdane", result.tlsDANE),
 				slog.Any("requiretls", m0.RequireTLS))
-			result = deliverHost(nqlog, resolver, dialer, ourHostname, transportName, h, enforceMTASTS, haveMX, origNextHopAuthentic, origNextHop, expandedNextHopAuthentic, expandedNextHop, msgResps, smtpclient.TLSSkip, false, &tlsrpt.Result{})
+			result = deliverHost(nqlog, resolver, dialer, ourHostname, transportName, transportDirect, h, enforceMTASTS, haveMX, origNextHopAuthentic, origNextHop, expandedNextHopAuthentic, expandedNextHop, msgResps, smtpclient.TLSSkip, false, &tlsrpt.Result{})
 		}
 
 		remoteMTA = dsn.NameIP{Name: h.XString(false), IP: remoteIP}
@@ -297,19 +299,39 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 			continue
 		}
 
-		delIDs := make([]int64, len(result.delivered))
+		delMsgs := make([]Msg, len(result.delivered))
 		for i, mr := range result.delivered {
 			mqlog := nqlog.With(slog.Int64("msgid", mr.msg.ID), slog.Any("recipient", mr.msg.Recipient()))
 			mqlog.Info("delivered from queue")
-			delIDs[i] = mr.msg.ID
+			mr.msg.markResult(0, "", "", true)
+			delMsgs[i] = *mr.msg
 		}
-		if len(delIDs) > 0 {
-			if err := queueDelete(context.Background(), delIDs...); err != nil {
-				nqlog.Errorx("deleting messages from queue after delivery", err)
+		if len(delMsgs) > 0 {
+			err := DB.Write(context.Background(), func(tx *bstore.Tx) error {
+				return retireMsgs(nqlog, tx, webhook.EventDelivered, 0, "", nil, delMsgs...)
+			})
+			if err != nil {
+				nqlog.Errorx("deleting messages from queue database after delivery", err)
+			} else if err := removeMsgsFS(nqlog, delMsgs...); err != nil {
+				nqlog.Errorx("removing queued messages from file system after delivery", err)
 			}
+			kick()
 		}
-		for _, mr := range result.failed {
-			fail(ctx, nqlog, []*Msg{mr.msg}, m0.DialedIPs, backoff, remoteMTA, smtpclient.Error(mr.resp))
+		if len(result.failed) > 0 {
+			err := DB.Write(context.Background(), func(tx *bstore.Tx) error {
+				for _, mr := range result.failed {
+					failMsgsTx(nqlog, tx, []*Msg{mr.msg}, m0.DialedIPs, backoff, remoteMTA, smtpclient.Error(mr.resp))
+				}
+				return nil
+			})
+			if err != nil {
+				for _, mr := range result.failed {
+					nqlog.Errorx("error processing delivery failure for messages", err,
+						slog.Int64("msgid", mr.msg.ID),
+						slog.Any("recipient", mr.msg.Recipient()))
+				}
+			}
+			kick()
 		}
 		return
 	}
@@ -334,11 +356,11 @@ func deliverDirect(qlog mlog.Log, resolver dns.Resolver, dialer smtpclient.Diale
 			Secode:    smtp.SePol7MissingReqTLS30,
 			Err:       fmt.Errorf("destination servers do not support requiretls"),
 		}
-		fail(ctx, qlog, msgs, m0.DialedIPs, backoff, remoteMTA, err)
+		failMsgsDB(qlog, msgs, m0.DialedIPs, backoff, remoteMTA, err)
 		return
 	}
 
-	fail(ctx, qlog, msgs, m0.DialedIPs, backoff, remoteMTA, lastErr)
+	failMsgsDB(qlog, msgs, m0.DialedIPs, backoff, remoteMTA, lastErr)
 	return
 }
 
@@ -375,7 +397,7 @@ type deliverResult struct {
 //
 // deliverHost may send a message multiple times: if the server doesn't accept
 // multiple recipients for a message.
-func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer, ourHostname dns.Domain, transportName string, host dns.IPDomain, enforceMTASTS, haveMX, origNextHopAuthentic bool, origNextHop dns.Domain, expandedNextHopAuthentic bool, expandedNextHop dns.Domain, msgResps []*msgResp, tlsMode smtpclient.TLSMode, tlsPKIX bool, recipientDomainResult *tlsrpt.Result) (result deliverResult) {
+func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer, ourHostname dns.Domain, transportName string, transportDirect *config.TransportDirect, host dns.IPDomain, enforceMTASTS, haveMX, origNextHopAuthentic bool, origNextHop dns.Domain, expandedNextHopAuthentic bool, expandedNextHop dns.Domain, msgResps []*msgResp, tlsMode smtpclient.TLSMode, tlsPKIX bool, recipientDomainResult *tlsrpt.Result) (result deliverResult) {
 	// About attempting delivery to multiple addresses of a host: ../rfc/5321:3898
 
 	m0 := msgResps[0].msg
@@ -451,7 +473,14 @@ func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer,
 	}
 
 	metricDestinations.Inc()
-	authentic, expandedAuthentic, expandedHost, ips, dualstack, err := smtpclient.GatherIPs(ctx, log.Logger, resolver, host, m0.DialedIPs)
+	network := "ip"
+	if transportDirect != nil {
+		if network != transportDirect.IPFamily {
+			log.Debug("set custom IP network family for direct transport", slog.Any("network", transportDirect.IPFamily))
+			network = transportDirect.IPFamily
+		}
+	}
+	authentic, expandedAuthentic, expandedHost, ips, dualstack, err := smtpclient.GatherIPs(ctx, log.Logger, resolver, network, host, m0.DialedIPs)
 	destAuthentic := err == nil && authentic && origNextHopAuthentic && (!haveMX || expandedNextHopAuthentic) && host.IsDomain()
 	if !destAuthentic {
 		log.Debugx("not attempting verification with dane", err, slog.Bool("authentic", authentic), slog.Bool("expandedauthentic", expandedAuthentic))
@@ -639,13 +668,16 @@ func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer,
 			err = fmt.Errorf("storing recipient domain tls status: %w", err)
 		}
 	}
-	if err != nil {
+
+	inspectError := func(err error) error {
 		if cerr, ok := err.(smtpclient.Error); ok {
 			// If we are being rejected due to policy reasons on the first
 			// attempt and remote has both IPv4 and IPv6, we'll give it
 			// another try. Our first IP may be in a block list, the address for
 			// the other family perhaps is not.
+
 			if cerr.Permanent && m0.Attempts == 1 && dualstack && strings.HasPrefix(cerr.Secode, "7.") {
+				log.Debugx("change error type from permanent to transient", err, slog.Any("host", host), slog.Any("secode", cerr.Secode))
 				cerr.Permanent = false
 			}
 			// If server does not implement requiretls, respond with that code. ../rfc/8689:301
@@ -653,9 +685,13 @@ func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer,
 				cerr.Secode = smtp.SePol7MissingReqTLS30
 				metricRequireTLSUnsupported.WithLabelValues("norequiretls").Inc()
 			}
-			err = cerr
+			return cerr
 		}
-		return deliverResult{err: err}
+		return err
+	}
+
+	if err != nil {
+		return deliverResult{err: inspectError(err)}
 	}
 
 	// SMTP session is ready. Finally try to actually deliver.
@@ -693,14 +729,18 @@ func deliverHost(log mlog.Log, resolver dns.Resolver, dialer smtpclient.Dialer,
 		}
 
 		resps, err := sc.DeliverMultiple(ctx, mailFrom, rcpts, size, msg, has8bit, smtputf8, m0.RequireTLS != nil && *m0.RequireTLS)
-		if err != nil && len(resps) == len(msgResps) {
+		if err != nil && (len(resps) == 0 && n == len(msgResps) || len(resps) == len(msgResps)) {
 			// If error and it applies to all recipients, return a single error.
-			return deliverResult{err: err}
+			return deliverResult{err: inspectError(err)}
 		}
 		var ntodo []*msgResp
 		for i, mr := range todo[:n] {
 			if err != nil {
-				mr.resp = smtpclient.Response{Err: err}
+				if cerr, ok := err.(smtpclient.Error); ok {
+					mr.resp = smtpclient.Response(cerr)
+				} else {
+					mr.resp = smtpclient.Response{Err: err}
+				}
 				failed = append(failed, mr)
 			} else if i > 0 && (resps[i].Code == smtp.C452StorageFull || resps[i].Code == smtp.C552MailboxFull) {
 				ntodo = append(ntodo, mr)
@@ -728,6 +768,10 @@ func updateRecipientDomainTLS(ctx context.Context, log mlog.Log, senderAccount s
 	if err != nil {
 		return fmt.Errorf("open account: %w", err)
 	}
+	defer func() {
+		err := acc.Close()
+		log.Check(err, "closing account")
+	}()
 	err = acc.DB.Write(ctx, func(tx *bstore.Tx) error {
 		// First delete any existing record.
 		if err := tx.Delete(&store.RecipientDomainTLS{Domain: rdt.Domain}); err != nil && err != bstore.ErrAbsent {
