@@ -86,7 +86,8 @@ func monitorDNSBL(log mlog.Log) {
 
 		// Gather zones.
 		zones := append([]dns.Domain{}, publicListener.SMTP.DNSBLZones...)
-		for _, zone := range mox.Conf.MonitorDNSBLs() {
+		conf := mox.Conf.DynamicConfig()
+		for _, zone := range conf.MonitorDNSBLZones {
 			if !slices.Contains(zones, zone) {
 				zones = append(zones, zone)
 			}
@@ -325,7 +326,7 @@ Only implemented on unix systems, not Windows.
 			}
 			defer store.CloseRemoveTempFile(log, f, "message for changelog delivery")
 
-			m := &store.Message{
+			m := store.Message{
 				Received: time.Now(),
 				Flags:    store.Flags{Flagged: true},
 			}
@@ -335,10 +336,16 @@ Only implemented on unix systems, not Windows.
 				return next
 			}
 			m.Size = int64(n)
-			if err := a.DeliverMailbox(log, mox.Conf.Static.Postmaster.Mailbox, m, f); err != nil {
-				log.Errorx("changelog delivery", err)
+
+			var derr error
+			a.WithWLock(func() {
+				derr = a.DeliverMailbox(log, mox.Conf.Static.Postmaster.Mailbox, &m, f)
+			})
+			if derr != nil {
+				log.Errorx("changelog delivery", derr)
 				return next
 			}
+
 			log.Info("delivered changelog",
 				slog.Any("current", current),
 				slog.Any("lastknown", lastknown),
