@@ -116,11 +116,13 @@ func newTestServer(t *testing.T, configPath string, resolver dns.Resolver) *test
 	mox.MustLoadConfig(true, false)
 	dataDir := mox.ConfigDirPath(mox.Conf.Static.DataDir)
 	os.RemoveAll(dataDir)
+
 	var err error
 	ts.acc, err = store.OpenAccount(log, "mjl")
 	tcheck(t, err, "open account")
 	err = ts.acc.SetPassword(log, password0)
 	tcheck(t, err, "set password")
+
 	ts.switchStop = store.Switchboard()
 	err = queue.Init()
 	tcheck(t, err, "queue init")
@@ -139,10 +141,29 @@ func (ts *testserver) close() {
 	ts.switchStop()
 	err := ts.acc.Close()
 	tcheck(ts.t, err, "closing account")
+	ts.acc.CheckClosed()
 	ts.acc = nil
 }
 
+func (ts *testserver) checkCount(mailboxName string, expect int) {
+	t := ts.t
+	t.Helper()
+	q := bstore.QueryDB[store.Mailbox](ctxbg, ts.acc.DB)
+	q.FilterNonzero(store.Mailbox{Name: mailboxName})
+	mb, err := q.Get()
+	tcheck(t, err, "get mailbox")
+	qm := bstore.QueryDB[store.Message](ctxbg, ts.acc.DB)
+	qm.FilterNonzero(store.Message{MailboxID: mb.ID})
+	qm.FilterEqual("Expunged", false)
+	n, err := qm.Count()
+	tcheck(t, err, "count messages in mailbox")
+	if n != expect {
+		t.Fatalf("messages in mailbox, found %d, expected %d", n, expect)
+	}
+}
+
 func (ts *testserver) run(fn func(helloErr error, client *smtpclient.Client)) {
+	ts.t.Helper()
 	ts.runRaw(func(conn net.Conn) {
 		ts.t.Helper()
 
@@ -192,6 +213,15 @@ func (ts *testserver) runRaw(fn func(clientConn net.Conn)) {
 	fn(clientConn)
 }
 
+func (ts *testserver) smtpErr(err error, expErr *smtpclient.Error) {
+	t := ts.t
+	t.Helper()
+	var cerr smtpclient.Error
+	if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Permanent != expErr.Permanent || cerr.Code != expErr.Code || cerr.Secode != expErr.Secode) {
+		t.Fatalf("got err:\n%#v (%q)\nexpected:\n%#v", err, err, expErr)
+	}
+}
+
 // Just a cert that appears valid. SMTP client will not verify anything about it
 // (that is opportunistic TLS for you, "better some than none"). Let's enjoy this
 // one moment where it makes life easier.
@@ -261,8 +291,8 @@ func TestSubmission(t *testing.T) {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(submitMessage)), strings.NewReader(submitMessage), false, false, false)
 			}
 			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v (%q), expected %#v", err, err, expErr)
+			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Code != expErr.Code || cerr.Secode != expErr.Secode) {
+				t.Fatalf("got err:\n%#v (%q)\nexpected:\n%#v", err, err, expErr)
 			}
 			checkEvaluationCount(t, 0)
 		})
@@ -288,8 +318,8 @@ func TestSubmission(t *testing.T) {
 		},
 	}
 	for _, fn := range authfns {
-		testAuth(fn, "mjl@mox.example", "test", &smtpclient.Error{Secode: smtp.SePol7AuthBadCreds8})           // Bad (short) password.
-		testAuth(fn, "mjl@mox.example", password0+"test", &smtpclient.Error{Secode: smtp.SePol7AuthBadCreds8}) // Bad password.
+		testAuth(fn, "mjl@mox.example", "test", &smtpclient.Error{Code: smtp.C535AuthBadCreds, Secode: smtp.SePol7AuthBadCreds8})           // Bad (short) password.
+		testAuth(fn, "mjl@mox.example", password0+"test", &smtpclient.Error{Code: smtp.C535AuthBadCreds, Secode: smtp.SePol7AuthBadCreds8}) // Bad password.
 		testAuth(fn, "mjl@mox.example", password0, nil)
 		testAuth(fn, "mjl@mox.example", password1, nil)
 		testAuth(fn, "móx@mox.example", password0, nil)
@@ -506,22 +536,6 @@ func TestSpam(t *testing.T) {
 		tinsertmsg(t, ts.acc, "Inbox", &nm, deliverMessage)
 	}
 
-	checkCount := func(mailboxName string, expect int) {
-		t.Helper()
-		q := bstore.QueryDB[store.Mailbox](ctxbg, ts.acc.DB)
-		q.FilterNonzero(store.Mailbox{Name: mailboxName})
-		mb, err := q.Get()
-		tcheck(t, err, "get rejects mailbox")
-		qm := bstore.QueryDB[store.Message](ctxbg, ts.acc.DB)
-		qm.FilterNonzero(store.Message{MailboxID: mb.ID})
-		qm.FilterEqual("Expunged", false)
-		n, err := qm.Count()
-		tcheck(t, err, "count messages in rejects mailbox")
-		if n != expect {
-			t.Fatalf("messages in rejects mailbox, found %d, expected %d", n, expect)
-		}
-	}
-
 	// Delivery from sender with bad reputation should fail.
 	ts.run(func(err error, client *smtpclient.Client) {
 		mailFrom := "remote@example.org"
@@ -534,7 +548,7 @@ func TestSpam(t *testing.T) {
 			t.Fatalf("delivery by bad sender, got err %v, expected smtpclient.Error with code %d", err, smtp.C451LocalErr)
 		}
 
-		checkCount("Rejects", 1)
+		ts.checkCount("Rejects", 1)
 		checkEvaluationCount(t, 0) // No positive interactions yet.
 	})
 
@@ -548,9 +562,9 @@ func TestSpam(t *testing.T) {
 		}
 		tcheck(t, err, "deliver")
 
-		checkCount("mjl2junk", 1)  // In ruleset rejects mailbox.
-		checkCount("Rejects", 1)   // Same as before.
-		checkEvaluationCount(t, 0) // This is not an actual accept.
+		ts.checkCount("mjl2junk", 1) // In ruleset rejects mailbox.
+		ts.checkCount("Rejects", 1)  // Same as before.
+		checkEvaluationCount(t, 0)   // This is not an actual accept.
 	})
 
 	// Mark the messages as having good reputation.
@@ -569,8 +583,8 @@ func TestSpam(t *testing.T) {
 		tcheck(t, err, "deliver")
 
 		// Message should now be removed from Rejects mailboxes.
-		checkCount("Rejects", 0)
-		checkCount("mjl2junk", 1)
+		ts.checkCount("Rejects", 0)
+		ts.checkCount("mjl2junk", 1)
 		checkEvaluationCount(t, 1)
 	})
 
@@ -1280,10 +1294,7 @@ func TestLimitOutgoing(t *testing.T) {
 			if err == nil {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(submitMessage)), strings.NewReader(submitMessage), false, false, false)
 			}
-			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v, expected %#v", err, expErr)
-			}
+			ts.smtpErr(err, expErr)
 		})
 	}
 
@@ -1317,10 +1328,7 @@ func TestQuota(t *testing.T) {
 			if err == nil {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(deliverMessage)), strings.NewReader(deliverMessage), false, false, false)
 			}
-			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v, expected %#v", err, expErr)
-			}
+			ts.smtpErr(err, expErr)
 		})
 	}
 
@@ -1348,10 +1356,7 @@ func TestCatchall(t *testing.T) {
 			if err == nil {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(submitMessage)), strings.NewReader(submitMessage), false, false, false)
 			}
-			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v, expected %#v", err, expErr)
-			}
+			ts.smtpErr(err, expErr)
 		})
 	}
 
@@ -1366,7 +1371,10 @@ func TestCatchall(t *testing.T) {
 
 	acc, err := store.OpenAccount(pkglog, "catchall")
 	tcheck(t, err, "open account")
-	defer acc.Close()
+	defer func() {
+		acc.Close()
+		acc.CheckClosed()
+	}()
 	n, err = bstore.QueryDB[store.Message](ctxbg, acc.DB).Count()
 	tcheck(t, err, "checking delivered messages to catchall account")
 	tcompare(t, n, 1)
@@ -1443,7 +1451,7 @@ test email
 			}
 			tcheck(t, err, "deliver")
 
-			msgs, err := queue.List(ctxbg)
+			msgs, err := queue.List(ctxbg, queue.Filter{}, queue.Sort{})
 			tcheck(t, err, "listing queue")
 			n++
 			tcompare(t, len(msgs), n)
@@ -1486,17 +1494,14 @@ func TestPostmaster(t *testing.T) {
 			if err == nil {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(deliverMessage)), strings.NewReader(deliverMessage), false, false, false)
 			}
-			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Code != expErr.Code || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v, expected %#v", err, expErr)
-			}
+			ts.smtpErr(err, expErr)
 		})
 	}
 
 	testDeliver("postmaster", nil)                  // Plain postmaster address without domain.
 	testDeliver("postmaster@host.mox.example", nil) // Postmaster address with configured mail server hostname.
 	testDeliver("postmaster@mox.example", nil)      // Postmaster address without explicitly configured destination.
-	testDeliver("postmaster@unknown.example", &smtpclient.Error{Code: smtp.C550MailboxUnavail, Secode: smtp.SeAddr1UnknownDestMailbox1})
+	testDeliver("postmaster@unknown.example", &smtpclient.Error{Permanent: true, Code: smtp.C550MailboxUnavail, Secode: smtp.SeAddr1UnknownDestMailbox1})
 }
 
 // Test to address with empty localpart.
@@ -1522,10 +1527,7 @@ func TestEmptylocalpart(t *testing.T) {
 			if err == nil {
 				err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(msg)), strings.NewReader(msg), false, false, false)
 			}
-			var cerr smtpclient.Error
-			if expErr == nil && err != nil || expErr != nil && (err == nil || !errors.As(err, &cerr) || cerr.Code != expErr.Code || cerr.Secode != expErr.Secode) {
-				t.Fatalf("got err %#v, expected %#v", err, expErr)
-			}
+			ts.smtpErr(err, expErr)
 		})
 	}
 
@@ -1592,11 +1594,11 @@ test email
 			}
 			tcheck(t, err, "deliver")
 
-			msgs, err := queue.List(ctxbg)
+			msgs, err := queue.List(ctxbg, queue.Filter{}, queue.Sort{})
 			tcheck(t, err, "listing queue")
 			tcompare(t, len(msgs), 1)
 			tcompare(t, msgs[0].RequireTLS, expRequireTLS)
-			_, err = queue.Drop(ctxbg, pkglog, msgs[0].ID, "", "")
+			_, err = queue.Drop(ctxbg, pkglog, queue.Filter{IDs: []int64{msgs[0].ID}})
 			tcheck(t, err, "deleting message from queue")
 		})
 	}
@@ -1767,3 +1769,177 @@ func TestFutureRelease(t *testing.T) {
 	test(" HOLDFOR=1 HOLDFOR=1", "501")                                                                              // Duplicate.
 	test(" HOLDFOR=1 HOLDUNTIL="+time.Now().Add(time.Hour).UTC().Format(time.RFC3339), "501")                        // Duplicate.
 }
+
+// Test SMTPUTF8
+func TestSMTPUTF8(t *testing.T) {
+	ts := newTestServer(t, filepath.FromSlash("../testdata/smtp/mox.conf"), dns.MockResolver{})
+	defer ts.close()
+
+	ts.user = "mjl@mox.example"
+	ts.pass = password0
+	ts.submission = true
+
+	test := func(mailFrom string, rcptTo string, headerValue string, filename string, clientSmtputf8 bool, expectedSmtputf8 bool, expErr *smtpclient.Error) {
+		t.Helper()
+
+		ts.run(func(_ error, client *smtpclient.Client) {
+			t.Helper()
+			msg := strings.ReplaceAll(fmt.Sprintf(`From: <%s>
+To: <%s>
+Subject: test
+X-Custom-Test-Header: %s
+MIME-Version: 1.0
+Content-type: multipart/mixed; boundary="simple boundary"
+
+--simple boundary
+Content-Type: text/plain; charset=UTF-8;
+Content-Disposition: attachment; filename="%s"
+Content-Transfer-Encoding: base64
+
+QW4gYXR0YWNoZWQgdGV4dCBmaWxlLg==
+
+--simple boundary--
+`, mailFrom, rcptTo, headerValue, filename), "\n", "\r\n")
+
+			err := client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(msg)), strings.NewReader(msg), true, clientSmtputf8, false)
+			ts.smtpErr(err, expErr)
+			if err != nil {
+				return
+			}
+
+			msgs, _ := queue.List(ctxbg, queue.Filter{}, queue.Sort{Field: "Queued", Asc: false})
+			queuedMsg := msgs[0]
+			if queuedMsg.SMTPUTF8 != expectedSmtputf8 {
+				t.Fatalf("[%s / %s / %s / %s] got SMTPUTF8 %t, expected %t", mailFrom, rcptTo, headerValue, filename, queuedMsg.SMTPUTF8, expectedSmtputf8)
+			}
+		})
+	}
+
+	test(`mjl@mox.example`, `remote@example.org`, "header-ascii", "ascii.txt", false, false, nil)
+	test(`mjl@mox.example`, `remote@example.org`, "header-ascii", "ascii.txt", true, false, nil)
+	test(`mjl@mox.example`, `🙂@example.org`, "header-ascii", "ascii.txt", true, true, nil)
+	test(`mjl@mox.example`, `🙂@example.org`, "header-ascii", "ascii.txt", false, true, &smtpclient.Error{Permanent: true, Code: smtp.C553BadMailbox, Secode: smtp.SeMsg6NonASCIIAddrNotPermitted7})
+	test(`Ω@mox.example`, `remote@example.org`, "header-ascii", "ascii.txt", true, true, nil)
+	test(`Ω@mox.example`, `remote@example.org`, "header-ascii", "ascii.txt", false, true, &smtpclient.Error{Permanent: true, Code: smtp.C550MailboxUnavail, Secode: smtp.SeMsg6NonASCIIAddrNotPermitted7})
+	test(`mjl@mox.example`, `remote@example.org`, "header-utf8-😍", "ascii.txt", true, true, nil)
+	test(`mjl@mox.example`, `remote@example.org`, "header-utf8-😍", "ascii.txt", false, true, nil)
+	test(`mjl@mox.example`, `remote@example.org`, "header-ascii", "utf8-🫠️.txt", true, true, nil)
+	test(`Ω@mox.example`, `🙂@example.org`, "header-utf8-😍", "utf8-🫠️.txt", true, true, nil)
+	test(`mjl@mox.example`, `remote@xn--vg8h.example.org`, "header-ascii", "ascii.txt", true, false, nil)
+}
+
+// TestExtra checks whether submission of messages with "X-Mox-Extra-<key>: value"
+// headers cause those those key/value pairs to be added to the Extra field in the
+// queue.
+func TestExtra(t *testing.T) {
+	ts := newTestServer(t, filepath.FromSlash("../testdata/smtp/mox.conf"), dns.MockResolver{})
+	defer ts.close()
+
+	ts.user = "mjl@mox.example"
+	ts.pass = password0
+	ts.submission = true
+
+	extraMsg := strings.ReplaceAll(`From: <mjl@mox.example>
+To: <remote@example.org>
+Subject: test
+X-Mox-Extra-Test: testvalue
+X-Mox-Extra-a: 123
+X-Mox-Extra-☺: ☹
+X-Mox-Extra-x-cANONICAL-z: ok
+Message-Id: <test@mox.example>
+
+test email
+`, "\n", "\r\n")
+
+	ts.run(func(err error, client *smtpclient.Client) {
+		t.Helper()
+		tcheck(t, err, "init client")
+		mailFrom := "mjl@mox.example"
+		rcptTo := "mjl@mox.example"
+		err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(extraMsg)), strings.NewReader(extraMsg), true, true, false)
+		tcheck(t, err, "deliver")
+	})
+	msgs, err := queue.List(ctxbg, queue.Filter{}, queue.Sort{})
+	tcheck(t, err, "queue list")
+	tcompare(t, len(msgs), 1)
+	tcompare(t, msgs[0].Extra, map[string]string{
+		"Test":          "testvalue",
+		"A":             "123",
+		"☺":             "☹",
+		"X-Canonical-Z": "ok",
+	})
+	// note: these headers currently stay in the message.
+}
+
+// TestExtraDup checks for an error for duplicate x-mox-extra-* keys.
+func TestExtraDup(t *testing.T) {
+	ts := newTestServer(t, filepath.FromSlash("../testdata/smtp/mox.conf"), dns.MockResolver{})
+	defer ts.close()
+
+	ts.user = "mjl@mox.example"
+	ts.pass = password0
+	ts.submission = true
+
+	extraMsg := strings.ReplaceAll(`From: <mjl@mox.example>
+To: <remote@example.org>
+Subject: test
+X-Mox-Extra-Test: testvalue
+X-Mox-Extra-Test: testvalue
+Message-Id: <test@mox.example>
+
+test email
+`, "\n", "\r\n")
+
+	ts.run(func(err error, client *smtpclient.Client) {
+		tcheck(t, err, "init client")
+		mailFrom := "mjl@mox.example"
+		rcptTo := "mjl@mox.example"
+		err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(extraMsg)), strings.NewReader(extraMsg), true, true, false)
+		ts.smtpErr(err, &smtpclient.Error{Permanent: true, Code: smtp.C554TransactionFailed, Secode: smtp.SeMsg6Other0})
+	})
+}
+
+// FromID can be specified during submission, but must be unique, with single recipient.
+func TestUniqueFromID(t *testing.T) {
+	ts := newTestServer(t, filepath.FromSlash("../testdata/smtpfromid/mox.conf"), dns.MockResolver{})
+	defer ts.close()
+
+	ts.user = "mjl+fromid@mox.example"
+	ts.pass = password0
+	ts.submission = true
+
+	extraMsg := strings.ReplaceAll(`From: <mjl@mox.example>
+To: <remote@example.org>
+Subject: test
+
+test email
+`, "\n", "\r\n")
+
+	// Specify our own unique id when queueing.
+	ts.run(func(err error, client *smtpclient.Client) {
+		tcheck(t, err, "init client")
+		mailFrom := "mjl+unique@mox.example"
+		rcptTo := "mjl@mox.example"
+		err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(extraMsg)), strings.NewReader(extraMsg), true, true, false)
+		ts.smtpErr(err, nil)
+	})
+
+	// But we can only use it once.
+	ts.run(func(err error, client *smtpclient.Client) {
+		tcheck(t, err, "init client")
+		mailFrom := "mjl+unique@mox.example"
+		rcptTo := "mjl@mox.example"
+		err = client.Deliver(ctxbg, mailFrom, rcptTo, int64(len(extraMsg)), strings.NewReader(extraMsg), true, true, false)
+		ts.smtpErr(err, &smtpclient.Error{Permanent: true, Code: smtp.C554TransactionFailed, Secode: smtp.SeAddr1SenderSyntax7})
+	})
+
+	// We cannot use our own fromid with multiple recipients.
+	ts.run(func(err error, client *smtpclient.Client) {
+		tcheck(t, err, "init client")
+		mailFrom := "mjl+unique2@mox.example"
+		rcptTo := []string{"mjl@mox.example", "mjl@mox.example"}
+		_, err = client.DeliverMultiple(ctxbg, mailFrom, rcptTo, int64(len(extraMsg)), strings.NewReader(extraMsg), true, true, false)
+		ts.smtpErr(err, &smtpclient.Error{Permanent: true, Code: smtp.C554TransactionFailed, Secode: smtp.SeProto5TooManyRcpts3})
+	})
+
+}
