@@ -485,8 +485,8 @@ type Message struct {
 	// filtering).
 	IsMailingList bool
 
-	// If this message is a DSN. For DSNs, we don't look at the subject when matching
-	// threads.
+	// If this message is a DSN, generated by us or received. For DSNs, we don't look
+	// at the subject when matching threads.
 	DSN bool
 
 	ReceivedTLSVersion     uint16 // 0 if unknown, 1 if plaintext/no TLS, otherwise TLS cipher suite.
@@ -719,8 +719,96 @@ type LoginSession struct {
 	csrfToken    CSRFToken
 }
 
+// Quoting is a setting for how to quote in replies/forwards.
+type Quoting string
+
+const (
+	Default Quoting = "" // Bottom-quote if text is selected, top-quote otherwise.
+	Bottom  Quoting = "bottom"
+	Top     Quoting = "top"
+)
+
+// Settings are webmail client settings.
+type Settings struct {
+	ID uint8 // Singleton ID 1.
+
+	Signature string
+	Quoting   Quoting
+
+	// Whether to show the bars underneath the address input fields indicating
+	// starttls/dnssec/dane/mtasts/requiretls support by address.
+	ShowAddressSecurity bool
+}
+
+// ViewMode how a message should be viewed: its text parts, html parts, or html
+// with loading external resources.
+type ViewMode string
+
+const (
+	ModeDefault ViewMode = ""
+	ModeText    ViewMode = "text"
+	ModeHTML    ViewMode = "html"
+	ModeHTMLExt ViewMode = "htmlext" // HTML with external resources.
+)
+
+// FromAddressSettings are webmail client settings per "From" address.
+type FromAddressSettings struct {
+	FromAddress string // Unicode.
+	ViewMode    ViewMode
+}
+
+// RulesetNoListID records a user "no" response to the question of
+// creating/removing a ruleset after moving a message with list-id header from/to
+// the inbox.
+type RulesetNoListID struct {
+	ID            int64
+	RcptToAddress string `bstore:"nonzero"`
+	ListID        string `bstore:"nonzero"`
+	ToInbox       bool   // Otherwise from Inbox to other mailbox.
+}
+
+// RulesetNoMsgFrom records a user "no" response to the question of
+// creating/moveing a ruleset after moving a mesage with message "from" address
+// from/to the inbox.
+type RulesetNoMsgFrom struct {
+	ID             int64
+	RcptToAddress  string `bstore:"nonzero"`
+	MsgFromAddress string `bstore:"nonzero"` // Unicode.
+	ToInbox        bool   // Otherwise from Inbox to other mailbox.
+}
+
+// RulesetNoMailbox represents a "never from/to this mailbox" response to the
+// question of adding/removing a ruleset after moving a message.
+type RulesetNoMailbox struct {
+	ID int64
+
+	// The mailbox from/to which the move has happened.
+	// Not a references, if mailbox is deleted, an entry becomes ineffective.
+	MailboxID int64 `bstore:"nonzero"`
+	ToMailbox bool  // Whether MailboxID is the destination of the move (instead of source).
+}
+
 // Types stored in DB.
-var DBTypes = []any{NextUIDValidity{}, Message{}, Recipient{}, Mailbox{}, Subscription{}, Outgoing{}, Password{}, Subjectpass{}, SyncState{}, Upgrade{}, RecipientDomainTLS{}, DiskUsage{}, LoginSession{}}
+var DBTypes = []any{
+	NextUIDValidity{},
+	Message{},
+	Recipient{},
+	Mailbox{},
+	Subscription{},
+	Outgoing{},
+	Password{},
+	Subjectpass{},
+	SyncState{},
+	Upgrade{},
+	RecipientDomainTLS{},
+	DiskUsage{},
+	LoginSession{},
+	Settings{},
+	FromAddressSettings{},
+	RulesetNoListID{},
+	RulesetNoMsgFrom{},
+	RulesetNoMailbox{},
+}
 
 // Account holds the information about a user, includings mailboxes, messages, imap subscriptions.
 type Account struct {
@@ -850,9 +938,15 @@ func OpenAccountDB(log mlog.Log, accountDir, accountName string) (a *Account, re
 		return acc, nil
 	}
 
-	// Ensure mailbox counts and total message size are set.
+	// Ensure singletons are present. Mailbox counts and total message size, Settings.
 	var mentioned bool
 	err = db.Write(context.TODO(), func(tx *bstore.Tx) error {
+		if tx.Get(&Settings{ID: 1}) == bstore.ErrAbsent {
+			if err := tx.Insert(&Settings{ID: 1, ShowAddressSecurity: true}); err != nil {
+				return err
+			}
+		}
+
 		err := bstore.QueryTx[Mailbox](tx).FilterEqual("HaveCounts", false).ForEach(func(mb Mailbox) error {
 			if !mentioned {
 				mentioned = true
@@ -886,7 +980,7 @@ func OpenAccountDB(log mlog.Log, accountDir, accountName string) (a *Account, re
 		return tx.Insert(&du)
 	})
 	if err != nil {
-		return nil, fmt.Errorf("calculating counts for mailbox: %v", err)
+		return nil, fmt.Errorf("calculating counts for mailbox or inserting settings: %v", err)
 	}
 
 	// Start adding threading if needed.
@@ -922,6 +1016,9 @@ func OpenAccountDB(log mlog.Log, accountDir, accountName string) (a *Account, re
 		defer func() {
 			err := closeAccount(acc)
 			log.Check(err, "closing use of account after upgrading account storage for threads", slog.String("account", a.Name))
+
+			// Mark that upgrade has finished, possibly error is indicated in threadsErr.
+			close(acc.threadsCompleted)
 		}()
 
 		defer func() {
@@ -932,9 +1029,6 @@ func OpenAccountDB(log mlog.Log, accountDir, accountName string) (a *Account, re
 				metrics.PanicInc(metrics.Upgradethreads)
 				acc.threadsErr = fmt.Errorf("panic during upgradeThreads: %v", x)
 			}
-
-			// Mark that upgrade has finished, possibly error is indicated in threadsErr.
-			close(acc.threadsCompleted)
 		}()
 
 		err := upgradeThreads(mox.Shutdown, log, acc, &up)
@@ -974,6 +1068,9 @@ func initAccount(db *bstore.DB) error {
 		if err := tx.Insert(&DiskUsage{ID: 1}); err != nil {
 			return err
 		}
+		if err := tx.Insert(&Settings{ID: 1}); err != nil {
+			return err
+		}
 
 		if len(mox.Conf.Static.DefaultMailboxes) > 0 {
 			// Deprecated in favor of InitialMailboxes.
@@ -1059,6 +1156,15 @@ func initAccount(db *bstore.DB) error {
 	})
 }
 
+// CheckClosed asserts that the account has a zero reference count. For use in tests.
+func (a *Account) CheckClosed() {
+	openAccounts.Lock()
+	defer openAccounts.Unlock()
+	if a.nused != 0 {
+		panic(fmt.Sprintf("account still in use, %d refs", a.nused))
+	}
+}
+
 // Close reduces the reference count, and closes the database connection when
 // it was the last user.
 func (a *Account) Close() error {
@@ -1256,7 +1362,8 @@ func (a *Account) HighestDeletedModSeq(tx *bstore.Tx) (ModSeq, error) {
 	return v.HighestDeletedModSeq, err
 }
 
-// WithWLock runs fn with account writelock held. Necessary for account/mailbox modification. For message delivery, a read lock is required.
+// WithWLock runs fn with account writelock held. Necessary for account/mailbox
+// modification. For message delivery, a read lock is required.
 func (a *Account) WithWLock(fn func()) {
 	a.Lock()
 	defer a.Unlock()
@@ -1395,7 +1502,7 @@ func (a *Account) DeliverMessage(log mlog.Log, tx *bstore.Tx, m *Message, msgFil
 		}
 	}
 
-	// todo: perhaps we should match the recipients based on smtp submission and a matching message-id? we now miss the addresses in bcc's. for webmail, we could insert the recipients directly.
+	// todo: perhaps we should match the recipients based on smtp submission and a matching message-id? we now miss the addresses in bcc's if the mail client doesn't save a message that includes the bcc header in the sent mailbox.
 	if mb.Sent && part != nil && part.Envelope != nil {
 		e := part.Envelope
 		sent := e.Date
@@ -1685,7 +1792,7 @@ func (a *Account) SubscriptionEnsure(tx *bstore.Tx, name string) ([]Change, erro
 	return []Change{ChangeAddSubscription{name, []string{`\NonExistent`}}}, nil
 }
 
-// MessageRuleset returns the first ruleset (if any) that message the message
+// MessageRuleset returns the first ruleset (if any) that matches the message
 // represented by msgPrefix and msgFile, with smtp and validation fields from m.
 func MessageRuleset(log mlog.Log, dest config.Destination, m *Message, msgPrefix []byte, msgFile *os.File) *config.Ruleset {
 	if len(dest.Rulesets) == 0 {
@@ -1713,6 +1820,11 @@ ruleset:
 				continue ruleset
 			}
 		}
+		if rs.MsgFromRegexpCompiled != nil {
+			if m.MsgFromLocalpart == "" && m.MsgFromDomain == "" || !rs.MsgFromRegexpCompiled.MatchString(m.MsgFromLocalpart.String()+"@"+m.MsgFromDomain) {
+				continue ruleset
+			}
+		}
 
 		if !rs.VerifiedDNSDomain.IsZero() {
 			d := rs.VerifiedDNSDomain.Name()
@@ -2140,8 +2252,8 @@ func OpenEmail(log mlog.Log, email string) (*Account, config.Destination, error)
 	if err != nil {
 		return nil, config.Destination{}, fmt.Errorf("%w: %v", ErrUnknownCredentials, err)
 	}
-	accountName, _, dest, err := mox.FindAccount(addr.Localpart, addr.Domain, false)
-	if err != nil && (errors.Is(err, mox.ErrAccountNotFound) || errors.Is(err, mox.ErrDomainNotFound)) {
+	accountName, _, _, dest, err := mox.LookupAddress(addr.Localpart, addr.Domain, false, false)
+	if err != nil && (errors.Is(err, mox.ErrAddressNotFound) || errors.Is(err, mox.ErrDomainNotFound)) {
 		return nil, config.Destination{}, ErrUnknownCredentials
 	} else if err != nil {
 		return nil, config.Destination{}, fmt.Errorf("looking up address: %v", err)
@@ -2215,6 +2327,32 @@ func (f Flags) Changed(other Flags) (mask Flags) {
 	return
 }
 
+// Strings returns the flags that are set in their string form.
+func (f Flags) Strings() []string {
+	fields := []struct {
+		word string
+		have bool
+	}{
+		{`$forwarded`, f.Forwarded},
+		{`$junk`, f.Junk},
+		{`$mdnsent`, f.MDNSent},
+		{`$notjunk`, f.Notjunk},
+		{`$phishing`, f.Phishing},
+		{`\answered`, f.Answered},
+		{`\deleted`, f.Deleted},
+		{`\draft`, f.Draft},
+		{`\flagged`, f.Flagged},
+		{`\seen`, f.Seen},
+	}
+	var l []string
+	for _, fh := range fields {
+		if fh.have {
+			l = append(l, fh.word)
+		}
+	}
+	return l
+}
+
 var systemWellKnownFlags = map[string]bool{
 	`\answered`:  true,
 	`\flagged`:   true,
