@@ -577,8 +577,8 @@ Period: %s - %s UTC
 	var queued bool
 	for _, rcpt := range recipients {
 		// If recipient is on suppression list, we won't queue the reporting message.
-		q := bstore.QueryDB[tlsrptdb.TLSRPTSuppressAddress](ctx, db)
-		q.FilterNonzero(tlsrptdb.TLSRPTSuppressAddress{ReportingAddress: rcpt.Address.Path().String()})
+		q := bstore.QueryDB[tlsrptdb.SuppressAddress](ctx, db)
+		q.FilterNonzero(tlsrptdb.SuppressAddress{ReportingAddress: rcpt.Address.Path().String()})
 		q.FilterGreater("Until", time.Now())
 		exists, err := q.Exists()
 		if err != nil {
@@ -589,7 +589,7 @@ Period: %s - %s UTC
 			continue
 		}
 
-		qm := queue.MakeMsg(from.Path(), rcpt.Address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now())
+		qm := queue.MakeMsg(from.Path(), rcpt.Address.Path(), has8bit, smtputf8, msgSize, messageID, []byte(msgPrefix), nil, time.Now(), subject)
 		// Don't try as long as regular deliveries, and stop before we would send the
 		// delayed DSN. Though we also won't send that due to IsTLSReport.
 		// ../rfc/8460:1077
@@ -620,7 +620,15 @@ Period: %s - %s UTC
 }
 
 func composeMessage(ctx context.Context, log mlog.Log, mf *os.File, policyDomain dns.Domain, confDKIM config.DKIM, fromAddr smtp.Address, recipients []message.NameAddress, subject, text, filename string, reportFile *os.File) (msgPrefix string, has8bit, smtputf8 bool, messageID string, rerr error) {
-	xc := message.NewComposer(mf, 100*1024*1024)
+	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
+	smtputf8 = fromAddr.Localpart.IsInternational()
+	for _, r := range recipients {
+		if smtputf8 {
+			smtputf8 = r.Address.Localpart.IsInternational()
+			break
+		}
+	}
+	xc := message.NewComposer(mf, 100*1024*1024, smtputf8)
 	defer func() {
 		x := recover()
 		if x == nil {
@@ -633,14 +641,6 @@ func composeMessage(ctx context.Context, log mlog.Log, mf *os.File, policyDomain
 		panic(x)
 	}()
 
-	// We only use smtputf8 if we have to, with a utf-8 localpart. For IDNA, we use ASCII domains.
-	for _, a := range recipients {
-		if a.Address.Localpart.IsInternational() {
-			xc.SMTPUTF8 = true
-			break
-		}
-	}
-
 	xc.HeaderAddrs("From", []message.NameAddress{{Address: fromAddr}})
 	xc.HeaderAddrs("To", recipients)
 	xc.Subject(subject)
@@ -662,7 +662,7 @@ func composeMessage(ctx context.Context, log mlog.Log, mf *os.File, policyDomain
 	xc.Line()
 
 	// Textual part, just mentioning this is a TLS report.
-	textBody, ct, cte := xc.TextPart(text)
+	textBody, ct, cte := xc.TextPart("plain", text)
 	textHdr := textproto.MIMEHeader{}
 	textHdr.Set("Content-Type", ct)
 	textHdr.Set("Content-Transfer-Encoding", cte)
