@@ -12,6 +12,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log/slog"
 	"os"
 	"path"
 	"runtime/debug"
@@ -21,6 +22,7 @@ import (
 	"time"
 
 	"golang.org/x/exp/maps"
+
 	"golang.org/x/text/unicode/norm"
 
 	"github.com/mjl-/bstore"
@@ -64,10 +66,10 @@ var importers = struct {
 
 // ImportManage should be run as a goroutine, it manages imports of mboxes/maildirs, propagating progress over SSE connections.
 func ImportManage() {
-	log := mlog.New("httpimport")
+	log := mlog.New("httpimport", nil)
 	defer func() {
 		if x := recover(); x != nil {
-			log.Error("import manage panic", mlog.Field("err", x))
+			log.Error("import manage panic", slog.Any("err", x))
 			debug.PrintStack()
 			metrics.PanicInc(metrics.Importmanage)
 		}
@@ -94,7 +96,7 @@ func ImportManage() {
 				sendEvent := func(kind string, v any) {
 					buf, err := json.Marshal(v)
 					if err != nil {
-						log.Errorx("marshal event", err, mlog.Field("kind", kind), mlog.Field("event", v))
+						log.Errorx("marshal event", err, slog.String("kind", kind), slog.Any("event", v))
 						return
 					}
 					ssemsg := fmt.Sprintf("event: %s\ndata: %s\n\n", kind, buf)
@@ -198,23 +200,22 @@ type importStep struct {
 }
 
 // importStart prepare the import and launches the goroutine to actually import.
-// importStart is responsible for closing f.
-func importStart(log *mlog.Log, accName string, f *os.File, skipMailboxPrefix string) (string, error) {
+// importStart is responsible for closing f and removing f.
+func importStart(log mlog.Log, accName string, f *os.File, skipMailboxPrefix string) (string, bool, error) {
 	defer func() {
 		if f != nil {
-			err := f.Close()
-			log.Check(err, "closing uploaded file")
+			store.CloseRemoveTempFile(log, f, "upload for import")
 		}
 	}()
 
 	buf := make([]byte, 16)
 	if _, err := cryptrand.Read(buf); err != nil {
-		return "", err
+		return "", false, err
 	}
 	token := fmt.Sprintf("%x", buf)
 
 	if _, err := f.Seek(0, 0); err != nil {
-		return "", fmt.Errorf("seek to start of file: %v", err)
+		return "", false, fmt.Errorf("seek to start of file: %v", err)
 	}
 
 	// Recognize file format.
@@ -223,12 +224,12 @@ func importStart(log *mlog.Log, accName string, f *os.File, skipMailboxPrefix st
 	magicGzip := []byte{0x1f, 0x8b}
 	magic := make([]byte, 4)
 	if _, err := f.ReadAt(magic, 0); err != nil {
-		return "", fmt.Errorf("detecting file format: %v", err)
+		return "", true, fmt.Errorf("detecting file format: %v", err)
 	}
 	if bytes.Equal(magic, magicZip) {
 		iszip = true
 	} else if !bytes.Equal(magic[:2], magicGzip) {
-		return "", fmt.Errorf("file is not a zip or gzip file")
+		return "", true, fmt.Errorf("file is not a zip or gzip file")
 	}
 
 	var zr *zip.Reader
@@ -236,23 +237,23 @@ func importStart(log *mlog.Log, accName string, f *os.File, skipMailboxPrefix st
 	if iszip {
 		fi, err := f.Stat()
 		if err != nil {
-			return "", fmt.Errorf("stat temporary import zip file: %v", err)
+			return "", false, fmt.Errorf("stat temporary import zip file: %v", err)
 		}
 		zr, err = zip.NewReader(f, fi.Size())
 		if err != nil {
-			return "", fmt.Errorf("opening zip file: %v", err)
+			return "", true, fmt.Errorf("opening zip file: %v", err)
 		}
 	} else {
 		gzr, err := gzip.NewReader(f)
 		if err != nil {
-			return "", fmt.Errorf("gunzip: %v", err)
+			return "", true, fmt.Errorf("gunzip: %v", err)
 		}
 		tr = tar.NewReader(gzr)
 	}
 
-	acc, err := store.OpenAccount(accName)
+	acc, err := store.OpenAccount(log, accName)
 	if err != nil {
-		return "", fmt.Errorf("open acount: %v", err)
+		return "", false, fmt.Errorf("open acount: %v", err)
 	}
 	acc.Lock() // Not using WithWLock because importMessage is responsible for unlocking.
 
@@ -261,7 +262,7 @@ func importStart(log *mlog.Log, accName string, f *os.File, skipMailboxPrefix st
 		acc.Unlock()
 		xerr := acc.Close()
 		log.Check(xerr, "closing account")
-		return "", fmt.Errorf("start transaction: %v", err)
+		return "", false, fmt.Errorf("start transaction: %v", err)
 	}
 
 	// Ensure token is registered before returning, with context that can be canceled.
@@ -270,14 +271,14 @@ func importStart(log *mlog.Log, accName string, f *os.File, skipMailboxPrefix st
 
 	log.Info("starting import")
 	go importMessages(ctx, log.WithCid(mox.Cid()), token, acc, tx, zr, tr, f, skipMailboxPrefix)
-	f = nil // importMessages is now responsible for closing.
+	f = nil // importMessages is now responsible for closing and removing.
 
-	return token, nil
+	return token, false, nil
 }
 
 // importMessages imports the messages from zip/tgz file f.
 // importMessages is responsible for unlocking and closing acc, and closing tx and f.
-func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store.Account, tx *bstore.Tx, zr *zip.Reader, tr *tar.Reader, f *os.File, skipMailboxPrefix string) {
+func importMessages(ctx context.Context, log mlog.Log, token string, acc *store.Account, tx *bstore.Tx, zr *zip.Reader, tr *tar.Reader, f *os.File, skipMailboxPrefix string) {
 	// If a fatal processing error occurs, we panic with this type.
 	type importError struct{ Err error }
 
@@ -290,7 +291,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 	sendEvent := func(kind string, v any) {
 		buf, err := json.Marshal(v)
 		if err != nil {
-			log.Errorx("marshal event", err, mlog.Field("kind", kind), mlog.Field("event", v))
+			log.Errorx("marshal event", err, slog.String("kind", kind), slog.Any("event", v))
 			return
 		}
 		ssemsg := fmt.Sprintf("event: %s\ndata: %s\n\n", kind, buf)
@@ -313,13 +314,12 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 	}
 
 	defer func() {
-		err := f.Close()
-		log.Check(err, "closing uploaded messages file")
+		store.CloseRemoveTempFile(log, f, "uploaded messages")
 
 		for _, id := range deliveredIDs {
 			p := acc.MessagePath(id)
 			err := os.Remove(p)
-			log.Check(err, "closing message file after import error", mlog.Field("path", p))
+			log.Check(err, "closing message file after import error", slog.String("path", p))
 		}
 		if tx != nil {
 			err := tx.Rollback()
@@ -340,7 +340,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 			problemf("%s (aborting)", err.Err)
 			sendEvent("aborted", importAborted{})
 		} else {
-			log.Error("import panic", mlog.Field("err", x))
+			log.Error("import panic", slog.Any("err", x))
 			debug.PrintStack()
 			metrics.PanicInc(metrics.Importmessages)
 		}
@@ -372,6 +372,12 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 	mailboxes := map[string]store.Mailbox{}
 	messages := map[string]int{}
 
+	maxSize := acc.QuotaMessageSize()
+	du := store.DiskUsage{ID: 1}
+	err = tx.Get(&du)
+	ximportcheckf(err, "get disk usage")
+	var addSize int64
+
 	// For maildirs, we are likely to get a possible dovecot-keywords file after having
 	// imported the messages. Once we see the keywords, we use them. But before that
 	// time we remember which messages miss a keywords. Once the keywords become
@@ -482,16 +488,20 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 
 	xdeliver := func(mb store.Mailbox, m *store.Message, f *os.File, pos string) {
 		defer func() {
-			if f != nil {
-				err := os.Remove(f.Name())
-				log.Check(err, "removing temporary message file for delivery")
-				err = f.Close()
-				log.Check(err, "closing temporary message file for delivery")
-			}
+			name := f.Name()
+			err = f.Close()
+			log.Check(err, "closing temporary message file for delivery")
+			err := os.Remove(name)
+			log.Check(err, "removing temporary message file for delivery")
 		}()
 		m.MailboxID = mb.ID
 		m.MailboxOrigID = mb.ID
 
+		addSize += m.Size
+		if maxSize > 0 && du.MessageSize+addSize > maxSize {
+			ximportcheckf(fmt.Errorf("account over maximum total size %d", maxSize), "checking quota")
+		}
+
 		if modseq == 0 {
 			var err error
 			modseq, err = acc.NextModSeq(tx)
@@ -514,7 +524,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 		}
 
 		// Parse message and store parsed information for later fast retrieval.
-		p, err := message.EnsurePart(log, false, f, m.Size)
+		p, err := message.EnsurePart(log.Logger, false, f, m.Size)
 		if err != nil {
 			problemf("parsing message %s: %s (continuing)", pos, err)
 		}
@@ -542,11 +552,11 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 			trainMessage(m, p, pos)
 		}
 
-		const consumeFile = true
 		const sync = false
 		const notrain = true
 		const nothreads = true
-		if err := acc.DeliverMessage(log, tx, m, f, consumeFile, sync, notrain, nothreads); err != nil {
+		const updateDiskUsage = false
+		if err := acc.DeliverMessage(log, tx, m, f, sync, notrain, nothreads, updateDiskUsage); err != nil {
 			problemf("delivering message %s: %s (continuing)", pos, err)
 			return
 		}
@@ -557,7 +567,6 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 			prevMailbox = mb.Name
 			sendEvent("count", importCount{mb.Name, messages[mb.Name]})
 		}
-		f = nil
 	}
 
 	ximportMbox := func(mailbox, filename string, r io.Reader) {
@@ -567,7 +576,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 		}
 		mb := xensureMailbox(mailbox)
 
-		mr := store.NewMboxReader(store.CreateMessageTemp, filename, r, log)
+		mr := store.NewMboxReader(log, store.CreateMessageTemp, filename, r)
 		for {
 			m, mf, pos, err := mr.Next()
 			if err == io.EOF {
@@ -587,14 +596,11 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 		}
 		mb := xensureMailbox(mailbox)
 
-		f, err := store.CreateMessageTemp("import")
+		f, err := store.CreateMessageTemp(log, "import")
 		ximportcheckf(err, "creating temp message")
 		defer func() {
 			if f != nil {
-				err := os.Remove(f.Name())
-				log.Check(err, "removing temporary file for delivery")
-				err = f.Close()
-				log.Check(err, "closing temporary file for delivery")
+				store.CloseRemoveTempFile(log, f, "message to import")
 			}
 		}()
 
@@ -715,7 +721,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 				mailbox := path.Dir(name)
 				dovecotKeywords := map[rune]string{}
 				words, err := store.ParseDovecotKeywordsFlags(r, log)
-				log.Check(err, "parsing dovecot keywords for mailbox", mlog.Field("mailbox", mailbox))
+				log.Check(err, "parsing dovecot keywords for mailbox", slog.String("mailbox", mailbox))
 				for i, kw := range words {
 					dovecotKeywords['a'+rune(i)] = kw
 				}
@@ -809,7 +815,7 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 	for _, count := range messages {
 		total += count
 	}
-	log.Debug("messages imported", mlog.Field("total", total))
+	log.Debug("messages imported", slog.Int("total", total))
 
 	// Send final update for count of last-imported mailbox.
 	if prevMailbox != "" {
@@ -845,6 +851,9 @@ func importMessages(ctx context.Context, log *mlog.Log, token string, acc *store
 		}
 	}
 
+	err = acc.AddMessageSize(log, tx, addSize)
+	ximportcheckf(err, "updating disk usage after import")
+
 	err = tx.Commit()
 	tx = nil
 	ximportcheckf(err, "commit")
