@@ -194,6 +194,11 @@ func xcheckf(ctx context.Context, err error, format string, args ...any) {
 	if err == nil {
 		return
 	}
+	// If caller tried saving a config that is invalid, or because of a bad request, cause a user error.
+	if errors.Is(err, mox.ErrConfig) || errors.Is(err, mox.ErrRequest) {
+		xcheckuserf(ctx, err, format, args...)
+	}
+
 	msg := fmt.Sprintf(format, args...)
 	errmsg := fmt.Sprintf("%s: %s", msg, err)
 	pkglog.WithContext(ctx).Errorx(msg, err)
@@ -1518,8 +1523,19 @@ func (Admin) ParseDomain(ctx context.Context, domain string) dns.Domain {
 	return d
 }
 
+// DomainConfig returns the configuration for a domain.
+func (Admin) DomainConfig(ctx context.Context, domain string) config.Domain {
+	d, err := dns.ParseDomain(domain)
+	xcheckuserf(ctx, err, "parse domain")
+	conf, ok := mox.Conf.Domain(d)
+	if !ok {
+		xcheckuserf(ctx, errors.New("no such domain"), "looking up domain")
+	}
+	return conf
+}
+
 // DomainLocalparts returns the encoded localparts and accounts configured in domain.
-func (Admin) DomainLocalparts(ctx context.Context, domain string) (localpartAccounts map[string]string) {
+func (Admin) DomainLocalparts(ctx context.Context, domain string) (localpartAccounts map[string]string, localpartAliases map[string]config.Alias) {
 	d, err := dns.ParseDomain(domain)
 	xcheckuserf(ctx, err, "parsing domain")
 	_, ok := mox.Conf.Domain(d)
@@ -1539,20 +1555,33 @@ func (Admin) Accounts(ctx context.Context) []string {
 }
 
 // Account returns the parsed configuration of an account.
-func (Admin) Account(ctx context.Context, account string) map[string]any {
-	ac, ok := mox.Conf.Account(account)
-	if !ok {
-		xcheckuserf(ctx, errors.New("no such account"), "looking up account")
+func (Admin) Account(ctx context.Context, account string) (accountConfig config.Account, diskUsage int64) {
+	log := pkglog.WithContext(ctx)
+
+	acc, err := store.OpenAccount(log, account)
+	if err != nil && errors.Is(err, store.ErrAccountUnknown) {
+		xcheckuserf(ctx, err, "looking up account")
 	}
+	xcheckf(ctx, err, "open account")
+	defer func() {
+		err := acc.Close()
+		log.Check(err, "closing account")
+	}()
 
-	// todo: should change sherpa to understand config.Account directly, with its anonymous structs.
-	buf, err := json.Marshal(ac)
-	xcheckf(ctx, err, "marshal to json")
-	r := map[string]any{}
-	err = json.Unmarshal(buf, &r)
-	xcheckf(ctx, err, "unmarshal from json")
+	var ac config.Account
+	acc.WithRLock(func() {
+		ac, _ = mox.Conf.Account(acc.Name)
 
-	return r
+		err := acc.DB.Read(ctx, func(tx *bstore.Tx) error {
+			du := store.DiskUsage{ID: 1}
+			err := tx.Get(&du)
+			diskUsage = du.MessageSize
+			return err
+		})
+		xcheckf(ctx, err, "get disk usage")
+	})
+
+	return ac, diskUsage
 }
 
 // ConfigFiles returns the paths and contents of the static and dynamic configuration files.
@@ -1574,7 +1603,7 @@ func (Admin) MTASTSPolicies(ctx context.Context) (records []mtastsdb.PolicyRecor
 // TLSReports returns TLS reports overlapping with period start/end, for the given
 // policy domain (or all domains if empty). The reports are sorted first by period
 // end (most recent first), then by policy domain.
-func (Admin) TLSReports(ctx context.Context, start, end time.Time, policyDomain string) (reports []tlsrptdb.TLSReportRecord) {
+func (Admin) TLSReports(ctx context.Context, start, end time.Time, policyDomain string) (reports []tlsrptdb.Record) {
 	var polDom dns.Domain
 	if policyDomain != "" {
 		var err error
@@ -1596,7 +1625,7 @@ func (Admin) TLSReports(ctx context.Context, start, end time.Time, policyDomain
 }
 
 // TLSReportID returns a single TLS report.
-func (Admin) TLSReportID(ctx context.Context, domain string, reportID int64) tlsrptdb.TLSReportRecord {
+func (Admin) TLSReportID(ctx context.Context, domain string, reportID int64) tlsrptdb.Record {
 	record, err := tlsrptdb.RecordID(ctx, reportID)
 	if err == nil && record.Domain != domain {
 		err = bstore.ErrAbsent
@@ -1784,7 +1813,8 @@ func dnsblsStatus(ctx context.Context, log mlog.Log, resolver dns.Resolver) (res
 	// todo: check health before using dnsbl?
 	using = mox.Conf.Static.Listeners["public"].SMTP.DNSBLZones
 	zones := append([]dns.Domain{}, using...)
-	for _, zone := range mox.Conf.MonitorDNSBLs() {
+	conf := mox.Conf.DynamicConfig()
+	for _, zone := range conf.MonitorDNSBLZones {
 		if !slices.Contains(zones, zone) {
 			zones = append(zones, zone)
 			monitoring = append(monitoring, zone)
@@ -1831,7 +1861,14 @@ func (Admin) MonitorDNSBLsSave(ctx context.Context, text string) {
 		}
 		zones = append(zones, d)
 	}
-	err := mox.MonitorDNSBLsSave(ctx, zones)
+
+	err := mox.ConfigSave(ctx, func(conf *config.Dynamic) {
+		conf.MonitorDNSBLs = make([]string, len(zones))
+		conf.MonitorDNSBLZones = nil
+		for i, z := range zones {
+			conf.MonitorDNSBLs[i] = z.Name()
+		}
+	})
 	xcheckf(ctx, err, "saving monitoring dnsbl zones")
 }
 
@@ -1937,10 +1974,15 @@ func (Admin) SetPassword(ctx context.Context, accountName, password string) {
 	xcheckf(ctx, err, "setting password")
 }
 
-// SetAccountLimits set new limits on outgoing messages for an account.
-func (Admin) SetAccountLimits(ctx context.Context, accountName string, maxOutgoingMessagesPerDay, maxFirstTimeRecipientsPerDay int, maxMsgSize int64) {
-	err := mox.AccountLimitsSave(ctx, accountName, maxOutgoingMessagesPerDay, maxFirstTimeRecipientsPerDay, maxMsgSize)
-	xcheckf(ctx, err, "saving account limits")
+// AccountSettingsSave set new settings for an account that only an admin can set.
+func (Admin) AccountSettingsSave(ctx context.Context, accountName string, maxOutgoingMessagesPerDay, maxFirstTimeRecipientsPerDay int, maxMsgSize int64, firstTimeSenderDelay bool) {
+	err := mox.AccountSave(ctx, accountName, func(acc *config.Account) {
+		acc.MaxOutgoingMessagesPerDay = maxOutgoingMessagesPerDay
+		acc.MaxFirstTimeRecipientsPerDay = maxFirstTimeRecipientsPerDay
+		acc.QuotaMessageSize = maxMsgSize
+		acc.NoFirstTimeSenderDelay = !firstTimeSenderDelay
+	})
+	xcheckf(ctx, err, "saving account settings")
 }
 
 // ClientConfigsDomain returns configurations for email clients, IMAP and
@@ -1954,13 +1996,6 @@ func (Admin) ClientConfigsDomain(ctx context.Context, domain string) mox.ClientC
 	return cc
 }
 
-// QueueList returns the messages currently in the outgoing queue.
-func (Admin) QueueList(ctx context.Context) []queue.Msg {
-	l, err := queue.List(ctx)
-	xcheckf(ctx, err, "listing messages in queue")
-	return l
-}
-
 // QueueSize returns the number of messages currently in the outgoing queue.
 func (Admin) QueueSize(ctx context.Context) int {
 	n, err := queue.Count(ctx)
@@ -1968,31 +2003,149 @@ func (Admin) QueueSize(ctx context.Context) int {
 	return n
 }
 
-// QueueKick initiates delivery of a message from the queue and sets the transport
-// to use for delivery.
-func (Admin) QueueKick(ctx context.Context, id int64, transport string) {
-	n, err := queue.Kick(ctx, id, "", "", &transport)
-	if err == nil && n == 0 {
-		err = errors.New("message not found")
-	}
-	xcheckf(ctx, err, "kick message in queue")
+// QueueHoldRuleList lists the hold rules.
+func (Admin) QueueHoldRuleList(ctx context.Context) []queue.HoldRule {
+	l, err := queue.HoldRuleList(ctx)
+	xcheckf(ctx, err, "listing queue hold rules")
+	return l
 }
 
-// QueueDrop removes a message from the queue.
-func (Admin) QueueDrop(ctx context.Context, id int64) {
+// QueueHoldRuleAdd adds a hold rule. Newly submitted and existing messages
+// matching the hold rule will be marked "on hold".
+func (Admin) QueueHoldRuleAdd(ctx context.Context, hr queue.HoldRule) queue.HoldRule {
+	var err error
+	hr.SenderDomain, err = dns.ParseDomain(hr.SenderDomainStr)
+	xcheckuserf(ctx, err, "parsing sender domain %q", hr.SenderDomainStr)
+	hr.RecipientDomain, err = dns.ParseDomain(hr.RecipientDomainStr)
+	xcheckuserf(ctx, err, "parsing recipient domain %q", hr.RecipientDomainStr)
+
 	log := pkglog.WithContext(ctx)
-	n, err := queue.Drop(ctx, log, id, "", "")
-	if err == nil && n == 0 {
-		err = errors.New("message not found")
-	}
-	xcheckf(ctx, err, "drop message from queue")
+	hr, err = queue.HoldRuleAdd(ctx, log, hr)
+	xcheckf(ctx, err, "adding queue hold rule")
+	return hr
 }
 
-// QueueSaveRequireTLS updates the requiretls field for a message in the queue,
-// to be used for the next delivery.
-func (Admin) QueueSaveRequireTLS(ctx context.Context, id int64, requireTLS *bool) {
-	err := queue.SaveRequireTLS(ctx, id, requireTLS)
-	xcheckf(ctx, err, "update requiretls for message in queue")
+// QueueHoldRuleRemove removes a hold rule. The Hold field of messages in
+// the queue are not changed.
+func (Admin) QueueHoldRuleRemove(ctx context.Context, holdRuleID int64) {
+	log := pkglog.WithContext(ctx)
+	err := queue.HoldRuleRemove(ctx, log, holdRuleID)
+	xcheckf(ctx, err, "removing queue hold rule")
+}
+
+// QueueList returns the messages currently in the outgoing queue.
+func (Admin) QueueList(ctx context.Context, filter queue.Filter, sort queue.Sort) []queue.Msg {
+	l, err := queue.List(ctx, filter, sort)
+	xcheckf(ctx, err, "listing messages in queue")
+	return l
+}
+
+// QueueNextAttemptSet sets a new time for next delivery attempt of matching
+// messages from the queue.
+func (Admin) QueueNextAttemptSet(ctx context.Context, filter queue.Filter, minutes int) (affected int) {
+	n, err := queue.NextAttemptSet(ctx, filter, time.Now().Add(time.Duration(minutes)*time.Minute))
+	xcheckf(ctx, err, "setting new next delivery attempt time for matching messages in queue")
+	return n
+}
+
+// QueueNextAttemptAdd adds a duration to the time of next delivery attempt of
+// matching messages from the queue.
+func (Admin) QueueNextAttemptAdd(ctx context.Context, filter queue.Filter, minutes int) (affected int) {
+	n, err := queue.NextAttemptAdd(ctx, filter, time.Duration(minutes)*time.Minute)
+	xcheckf(ctx, err, "adding duration to next delivery attempt for matching messages in queue")
+	return n
+}
+
+// QueueHoldSet sets the Hold field of matching messages in the queue.
+func (Admin) QueueHoldSet(ctx context.Context, filter queue.Filter, onHold bool) (affected int) {
+	n, err := queue.HoldSet(ctx, filter, onHold)
+	xcheckf(ctx, err, "changing onhold for matching messages in queue")
+	return n
+}
+
+// QueueFail fails delivery for matching messages, causing DSNs to be sent.
+func (Admin) QueueFail(ctx context.Context, filter queue.Filter) (affected int) {
+	log := pkglog.WithContext(ctx)
+	n, err := queue.Fail(ctx, log, filter)
+	xcheckf(ctx, err, "drop messages from queue")
+	return n
+}
+
+// QueueDrop removes matching messages from the queue.
+func (Admin) QueueDrop(ctx context.Context, filter queue.Filter) (affected int) {
+	log := pkglog.WithContext(ctx)
+	n, err := queue.Drop(ctx, log, filter)
+	xcheckf(ctx, err, "drop messages from queue")
+	return n
+}
+
+// QueueRequireTLSSet updates the requiretls field for matching messages in the
+// queue, to be used for the next delivery.
+func (Admin) QueueRequireTLSSet(ctx context.Context, filter queue.Filter, requireTLS *bool) (affected int) {
+	n, err := queue.RequireTLSSet(ctx, filter, requireTLS)
+	xcheckf(ctx, err, "update requiretls for messages in queue")
+	return n
+}
+
+// QueueTransportSet initiates delivery of a message from the queue and sets the transport
+// to use for delivery.
+func (Admin) QueueTransportSet(ctx context.Context, filter queue.Filter, transport string) (affected int) {
+	n, err := queue.TransportSet(ctx, filter, transport)
+	xcheckf(ctx, err, "changing transport for messages in queue")
+	return n
+}
+
+// RetiredList returns messages retired from the queue (delivery could
+// have succeeded or failed).
+func (Admin) RetiredList(ctx context.Context, filter queue.RetiredFilter, sort queue.RetiredSort) []queue.MsgRetired {
+	l, err := queue.RetiredList(ctx, filter, sort)
+	xcheckf(ctx, err, "listing retired messages")
+	return l
+}
+
+// HookQueueSize returns the number of webhooks still to be delivered.
+func (Admin) HookQueueSize(ctx context.Context) int {
+	n, err := queue.HookQueueSize(ctx)
+	xcheckf(ctx, err, "get hook queue size")
+	return n
+}
+
+// HookList lists webhooks still to be delivered.
+func (Admin) HookList(ctx context.Context, filter queue.HookFilter, sort queue.HookSort) []queue.Hook {
+	l, err := queue.HookList(ctx, filter, sort)
+	xcheckf(ctx, err, "listing hook queue")
+	return l
+}
+
+// HookNextAttemptSet sets a new time for next delivery attempt of matching
+// hooks from the queue.
+func (Admin) HookNextAttemptSet(ctx context.Context, filter queue.HookFilter, minutes int) (affected int) {
+	n, err := queue.HookNextAttemptSet(ctx, filter, time.Now().Add(time.Duration(minutes)*time.Minute))
+	xcheckf(ctx, err, "setting new next delivery attempt time for matching webhooks in queue")
+	return n
+}
+
+// HookNextAttemptAdd adds a duration to the time of next delivery attempt of
+// matching hooks from the queue.
+func (Admin) HookNextAttemptAdd(ctx context.Context, filter queue.HookFilter, minutes int) (affected int) {
+	n, err := queue.HookNextAttemptAdd(ctx, filter, time.Duration(minutes)*time.Minute)
+	xcheckf(ctx, err, "adding duration to next delivery attempt for matching webhooks in queue")
+	return n
+}
+
+// HookRetiredList lists retired webhooks.
+func (Admin) HookRetiredList(ctx context.Context, filter queue.HookRetiredFilter, sort queue.HookRetiredSort) []queue.HookRetired {
+	l, err := queue.HookRetiredList(ctx, filter, sort)
+	xcheckf(ctx, err, "listing retired hooks")
+	return l
+}
+
+// HookCancel prevents further delivery attempts of matching webhooks.
+func (Admin) HookCancel(ctx context.Context, filter queue.HookFilter) (affected int) {
+	log := pkglog.WithContext(ctx)
+	n, err := queue.HookCancel(ctx, log, filter)
+	xcheckf(ctx, err, "cancel hooks in queue")
+	return n
 }
 
 // LogLevels returns the current log levels.
@@ -2043,7 +2196,10 @@ func (Admin) WebserverConfig(ctx context.Context) (conf WebserverConfig) {
 }
 
 func webserverConfig() WebserverConfig {
-	r, l := mox.Conf.WebServer()
+	conf := mox.Conf.DynamicConfig()
+	r := conf.WebDNSDomainRedirects
+	l := conf.WebHandlers
+
 	x := make([][2]dns.Domain, 0, len(r))
 	xs := make([][2]string, 0, len(r))
 	for k, v := range r {
@@ -2089,7 +2245,10 @@ func (Admin) WebserverConfigSave(ctx context.Context, oldConf, newConf Webserver
 		domainRedirects[x[0]] = x[1]
 	}
 
-	err := mox.WebserverConfigSet(ctx, domainRedirects, newConf.WebHandlers)
+	err := mox.ConfigSave(ctx, func(conf *config.Dynamic) {
+		conf.WebDomainRedirects = domainRedirects
+		conf.WebHandlers = newConf.WebHandlers
+	})
 	xcheckf(ctx, err, "saving webserver config")
 
 	savedConf = webserverConfig()
@@ -2225,13 +2384,13 @@ func (Admin) TLSRPTSuppressAdd(ctx context.Context, reportingAddress string, unt
 	addr, err := smtp.ParseAddress(reportingAddress)
 	xcheckuserf(ctx, err, "parsing reporting address")
 
-	ba := tlsrptdb.TLSRPTSuppressAddress{ReportingAddress: addr.String(), Until: until, Comment: comment}
+	ba := tlsrptdb.SuppressAddress{ReportingAddress: addr.String(), Until: until, Comment: comment}
 	err = tlsrptdb.SuppressAdd(ctx, &ba)
 	xcheckf(ctx, err, "adding address to suppresslist")
 }
 
 // TLSRPTSuppressList returns all reporting addresses on the suppress list.
-func (Admin) TLSRPTSuppressList(ctx context.Context) []tlsrptdb.TLSRPTSuppressAddress {
+func (Admin) TLSRPTSuppressList(ctx context.Context) []tlsrptdb.SuppressAddress {
 	l, err := tlsrptdb.SuppressList(ctx)
 	xcheckf(ctx, err, "listing reporting addresses in suppresslist")
 	return l
@@ -2255,3 +2414,234 @@ func (Admin) LookupCid(ctx context.Context, recvID string) (cid string) {
 	xcheckf(ctx, err, "received id to cid")
 	return fmt.Sprintf("%x", v)
 }
+
+// Config returns the dynamic config.
+func (Admin) Config(ctx context.Context) config.Dynamic {
+	return mox.Conf.DynamicConfig()
+}
+
+// AccountRoutesSave saves routes for an account.
+func (Admin) AccountRoutesSave(ctx context.Context, accountName string, routes []config.Route) {
+	err := mox.AccountSave(ctx, accountName, func(acc *config.Account) {
+		acc.Routes = routes
+	})
+	xcheckf(ctx, err, "saving account routes")
+}
+
+// DomainRoutesSave saves routes for a domain.
+func (Admin) DomainRoutesSave(ctx context.Context, domainName string, routes []config.Route) {
+	err := mox.DomainSave(ctx, domainName, func(domain *config.Domain) error {
+		domain.Routes = routes
+		return nil
+	})
+	xcheckf(ctx, err, "saving domain routes")
+}
+
+// RoutesSave saves global routes.
+func (Admin) RoutesSave(ctx context.Context, routes []config.Route) {
+	err := mox.ConfigSave(ctx, func(config *config.Dynamic) {
+		config.Routes = routes
+	})
+	xcheckf(ctx, err, "saving global routes")
+}
+
+// DomainDescriptionSave saves the description for a domain.
+func (Admin) DomainDescriptionSave(ctx context.Context, domainName, descr string) {
+	err := mox.DomainSave(ctx, domainName, func(domain *config.Domain) error {
+		domain.Description = descr
+		return nil
+	})
+	xcheckf(ctx, err, "saving domain description")
+}
+
+// DomainClientSettingsDomainSave saves the client settings domain for a domain.
+func (Admin) DomainClientSettingsDomainSave(ctx context.Context, domainName, clientSettingsDomain string) {
+	err := mox.DomainSave(ctx, domainName, func(domain *config.Domain) error {
+		domain.ClientSettingsDomain = clientSettingsDomain
+		return nil
+	})
+	xcheckf(ctx, err, "saving client settings domain")
+}
+
+// DomainLocalpartConfigSave saves the localpart catchall and case-sensitive
+// settings for a domain.
+func (Admin) DomainLocalpartConfigSave(ctx context.Context, domainName, localpartCatchallSeparator string, localpartCaseSensitive bool) {
+	err := mox.DomainSave(ctx, domainName, func(domain *config.Domain) error {
+		domain.LocalpartCatchallSeparator = localpartCatchallSeparator
+		domain.LocalpartCaseSensitive = localpartCaseSensitive
+		return nil
+	})
+	xcheckf(ctx, err, "saving localpart settings for domain")
+}
+
+// DomainDMARCAddressSave saves the DMARC reporting address/processing
+// configuration for a domain. If localpart is empty, processing reports is
+// disabled.
+func (Admin) DomainDMARCAddressSave(ctx context.Context, domainName, localpart, domain, account, mailbox string) {
+	err := mox.DomainSave(ctx, domainName, func(d *config.Domain) error {
+		if localpart == "" {
+			d.DMARC = nil
+		} else {
+			d.DMARC = &config.DMARC{
+				Localpart: localpart,
+				Domain:    domain,
+				Account:   account,
+				Mailbox:   mailbox,
+			}
+		}
+		return nil
+	})
+	xcheckf(ctx, err, "saving dmarc reporting address/settings for domain")
+}
+
+// DomainTLSRPTAddressSave saves the TLS reporting address/processing
+// configuration for a domain. If localpart is empty, processing reports is
+// disabled.
+func (Admin) DomainTLSRPTAddressSave(ctx context.Context, domainName, localpart, domain, account, mailbox string) {
+	err := mox.DomainSave(ctx, domainName, func(d *config.Domain) error {
+		if localpart == "" {
+			d.TLSRPT = nil
+		} else {
+			d.TLSRPT = &config.TLSRPT{
+				Localpart: localpart,
+				Domain:    domain,
+				Account:   account,
+				Mailbox:   mailbox,
+			}
+		}
+		return nil
+	})
+	xcheckf(ctx, err, "saving tls reporting address/settings for domain")
+}
+
+// DomainMTASTSSave saves the MTASTS policy for a domain. If policyID is empty,
+// no MTASTS policy is served.
+func (Admin) DomainMTASTSSave(ctx context.Context, domainName, policyID string, mode mtasts.Mode, maxAge time.Duration, mx []string) {
+	err := mox.DomainSave(ctx, domainName, func(d *config.Domain) error {
+		if policyID == "" {
+			d.MTASTS = nil
+		} else {
+			d.MTASTS = &config.MTASTS{
+				PolicyID: policyID,
+				Mode:     mode,
+				MaxAge:   maxAge,
+				MX:       mx,
+			}
+		}
+		return nil
+	})
+	xcheckf(ctx, err, "saving mtasts policy for domain")
+}
+
+// DomainDKIMAdd adds a DKIM selector for a domain, generating a new private
+// key. The selector is not enabled for signing.
+func (Admin) DomainDKIMAdd(ctx context.Context, domainName, selector, algorithm, hash string, headerRelaxed, bodyRelaxed, seal bool, headers []string, lifetime time.Duration) {
+	d, err := dns.ParseDomain(domainName)
+	xcheckuserf(ctx, err, "parsing domain")
+	s, err := dns.ParseDomain(selector)
+	xcheckuserf(ctx, err, "parsing selector")
+	err = mox.DKIMAdd(ctx, d, s, algorithm, hash, headerRelaxed, bodyRelaxed, seal, headers, lifetime)
+	xcheckf(ctx, err, "adding dkim key")
+}
+
+// DomainDKIMRemove removes a DKIM selector for a domain.
+func (Admin) DomainDKIMRemove(ctx context.Context, domainName, selector string) {
+	d, err := dns.ParseDomain(domainName)
+	xcheckuserf(ctx, err, "parsing domain")
+	s, err := dns.ParseDomain(selector)
+	xcheckuserf(ctx, err, "parsing selector")
+	err = mox.DKIMRemove(ctx, d, s)
+	xcheckf(ctx, err, "removing dkim key")
+}
+
+// DomainDKIMSave saves the settings of selectors, and which to enable for
+// signing, for a domain. All currently configured selectors must be present,
+// selectors cannot be added/removed with this function.
+func (Admin) DomainDKIMSave(ctx context.Context, domainName string, selectors map[string]config.Selector, sign []string) {
+	for _, s := range sign {
+		if _, ok := selectors[s]; !ok {
+			xcheckuserf(ctx, fmt.Errorf("cannot sign unknown selector %q", s), "checking selectors")
+		}
+	}
+
+	err := mox.DomainSave(ctx, domainName, func(d *config.Domain) error {
+		if len(selectors) != len(d.DKIM.Selectors) {
+			xcheckuserf(ctx, fmt.Errorf("cannot add/remove dkim selectors with this function"), "checking selectors")
+		}
+		for s := range selectors {
+			if _, ok := d.DKIM.Selectors[s]; !ok {
+				xcheckuserf(ctx, fmt.Errorf("unknown selector %q", s), "checking selectors")
+			}
+		}
+		// At least the selectors are the same.
+
+		// Build up new selectors.
+		sels := map[string]config.Selector{}
+		for name, nsel := range selectors {
+			osel := d.DKIM.Selectors[name]
+			xsel := config.Selector{
+				Hash:             nsel.Hash,
+				Canonicalization: nsel.Canonicalization,
+				DontSealHeaders:  nsel.DontSealHeaders,
+				Expiration:       nsel.Expiration,
+
+				PrivateKeyFile: osel.PrivateKeyFile,
+			}
+			if !slices.Equal(osel.HeadersEffective, nsel.Headers) {
+				xsel.Headers = nsel.Headers
+			}
+			sels[name] = xsel
+		}
+
+		// Enable the new selector settings.
+		d.DKIM = config.DKIM{
+			Selectors: sels,
+			Sign:      sign,
+		}
+		return nil
+	})
+	xcheckf(ctx, err, "saving dkim selector for domain")
+}
+
+func xparseAddress(ctx context.Context, lp, domain string) smtp.Address {
+	xlp, err := smtp.ParseLocalpart(lp)
+	xcheckuserf(ctx, err, "parsing localpart")
+	d, err := dns.ParseDomain(domain)
+	xcheckuserf(ctx, err, "parsing domain")
+	return smtp.NewAddress(xlp, d)
+}
+
+func (Admin) AliasAdd(ctx context.Context, aliaslp string, domainName string, alias config.Alias) {
+	addr := xparseAddress(ctx, aliaslp, domainName)
+	err := mox.AliasAdd(ctx, addr, alias)
+	xcheckf(ctx, err, "adding alias")
+}
+
+func (Admin) AliasUpdate(ctx context.Context, aliaslp string, domainName string, postPublic, listMembers, allowMsgFrom bool) {
+	addr := xparseAddress(ctx, aliaslp, domainName)
+	alias := config.Alias{
+		PostPublic:   postPublic,
+		ListMembers:  listMembers,
+		AllowMsgFrom: allowMsgFrom,
+	}
+	err := mox.AliasUpdate(ctx, addr, alias)
+	xcheckf(ctx, err, "saving alias")
+}
+
+func (Admin) AliasRemove(ctx context.Context, aliaslp string, domainName string) {
+	addr := xparseAddress(ctx, aliaslp, domainName)
+	err := mox.AliasRemove(ctx, addr)
+	xcheckf(ctx, err, "removing alias")
+}
+
+func (Admin) AliasAddressesAdd(ctx context.Context, aliaslp string, domainName string, addresses []string) {
+	addr := xparseAddress(ctx, aliaslp, domainName)
+	err := mox.AliasAddressesAdd(ctx, addr, addresses)
+	xcheckf(ctx, err, "adding address to alias")
+}
+
+func (Admin) AliasAddressesRemove(ctx context.Context, aliaslp string, domainName string, addresses []string) {
+	addr := xparseAddress(ctx, aliaslp, domainName)
+	err := mox.AliasAddressesRemove(ctx, addr, addresses)
+	xcheckf(ctx, err, "removing address from alias")
+}
