@@ -53,6 +53,7 @@ func TestAPI(t *testing.T) {
 	os.RemoveAll("../testdata/webmail/data")
 	mox.Context = ctxbg
 	mox.ConfigStaticPath = filepath.FromSlash("../testdata/webmail/mox.conf")
+	mox.ConfigDynamicPath = filepath.FromSlash("../testdata/webmail/domains.conf")
 	mox.MustLoadConfig(true, false)
 	defer store.Switchboard()()
 
@@ -66,6 +67,7 @@ func TestAPI(t *testing.T) {
 	defer func() {
 		err := acc.Close()
 		pkglog.Check(err, "closing account")
+		acc.CheckClosed()
 	}()
 
 	var zerom store.Message
@@ -88,7 +90,7 @@ func TestAPI(t *testing.T) {
 	api := Webmail{maxMessageSize: 1024 * 1024, cookiePath: "/webmail/"}
 
 	// Test login, and rate limiter.
-	loginReqInfo := requestInfo{"mjl@mox.example", "mjl", "", httptest.NewRecorder(), &http.Request{RemoteAddr: "1.1.1.1:1234"}}
+	loginReqInfo := requestInfo{log, "mjl@mox.example", nil, "", httptest.NewRecorder(), &http.Request{RemoteAddr: "1.1.1.1:1234"}}
 	loginctx := context.WithValue(ctxbg, requestInfoCtxKey, loginReqInfo)
 
 	// Missing login token.
@@ -136,7 +138,7 @@ func TestAPI(t *testing.T) {
 	testLogin("bad@bad.example", pw0, "user:error")
 
 	// Context with different IP, for clear rate limit history.
-	reqInfo := requestInfo{"mjl@mox.example", "mjl", "", nil, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
+	reqInfo := requestInfo{log, "mjl@mox.example", acc, "", nil, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx := context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	// FlagsAdd
@@ -174,14 +176,14 @@ func TestAPI(t *testing.T) {
 	tneedError(t, func() { api.FlagsClear(ctx, []int64{inboxText.ID}, []string{`\unknownsystem`}) })
 
 	// MailboxSetSpecialUse
-	var inbox, archive, sent, testbox1 store.Mailbox
+	var inbox, archive, sent, drafts, testbox1 store.Mailbox
 	err = acc.DB.Read(ctx, func(tx *bstore.Tx) error {
 		get := func(k string, v any) store.Mailbox {
 			mb, err := bstore.QueryTx[store.Mailbox](tx).FilterEqual(k, v).Get()
 			tcheck(t, err, "get special-use mailbox")
 			return mb
 		}
-		get("Draft", true)
+		drafts = get("Draft", true)
 		sent = get("Sent", true)
 		archive = get("Archive", true)
 		get("Trash", true)
@@ -218,13 +220,18 @@ func TestAPI(t *testing.T) {
 	// ParsedMessage
 	// todo: verify contents
 	api.ParsedMessage(ctx, inboxMinimal.ID)
-	api.ParsedMessage(ctx, inboxText.ID)
 	api.ParsedMessage(ctx, inboxHTML.ID)
 	api.ParsedMessage(ctx, inboxAlt.ID)
 	api.ParsedMessage(ctx, inboxAltRel.ID)
 	api.ParsedMessage(ctx, testbox1Alt.ID)
 	tneedError(t, func() { api.ParsedMessage(ctx, 0) })
 	tneedError(t, func() { api.ParsedMessage(ctx, testmsgs[len(testmsgs)-1].ID+1) })
+	pm := api.ParsedMessage(ctx, inboxText.ID)
+	tcompare(t, pm.ViewMode, store.ModeDefault)
+
+	api.FromAddressSettingsSave(ctx, store.FromAddressSettings{FromAddress: "mjl@mox.example", ViewMode: store.ModeHTMLExt})
+	pm = api.ParsedMessage(ctx, inboxText.ID)
+	tcompare(t, pm.ViewMode, store.ModeHTMLExt)
 
 	// MailboxDelete
 	api.MailboxDelete(ctx, testbox1.ID)
@@ -272,17 +279,46 @@ func TestAPI(t *testing.T) {
 	tdeliver(t, acc, testbox1Alt)
 	tdeliver(t, acc, inboxAltRel)
 
+	// MessageCompose
+	draftID := api.MessageCompose(ctx, ComposeMessage{
+		From:     "mjl@mox.example",
+		To:       []string{"mjl+to@mox.example", "mjl to2 <mjl+to2@mox.example>"},
+		Cc:       []string{"mjl+cc@mox.example", "mjl cc2 <mjl+cc2@mox.example>"},
+		Bcc:      []string{"mjl+bcc@mox.example", "mjl bcc2 <mjl+bcc2@mox.example>"},
+		Subject:  "test email",
+		TextBody: "this is the content\n\ncheers,\nmox",
+		ReplyTo:  "mjl replyto <mjl+replyto@mox.example>",
+	}, drafts.ID)
+	// Replace draft.
+	draftID = api.MessageCompose(ctx, ComposeMessage{
+		From:           "mjl@mox.example",
+		To:             []string{"mjl+to@mox.example", "mjl to2 <mjl+to2@mox.example>"},
+		Cc:             []string{"mjl+cc@mox.example", "mjl cc2 <mjl+cc2@mox.example>"},
+		Bcc:            []string{"mjl+bcc@mox.example", "mjl bcc2 <mjl+bcc2@mox.example>"},
+		Subject:        "test email",
+		TextBody:       "this is the content\n\ncheers,\nmox",
+		ReplyTo:        "mjl replyto <mjl+replyto@mox.example>",
+		DraftMessageID: draftID,
+	}, drafts.ID)
+
+	// MessageFindMessageID
+	msgID := api.MessageFindMessageID(ctx, "<absent@localhost>")
+	tcompare(t, msgID, int64(0))
+
 	// MessageSubmit
 	queue.Localserve = true // Deliver directly to us instead attempting actual delivery.
+	err = queue.Init()
+	tcheck(t, err, "queue init")
 	api.MessageSubmit(ctx, SubmitMessage{
-		From:      "mjl@mox.example",
-		To:        []string{"mjl+to@mox.example", "mjl to2 <mjl+to2@mox.example>"},
-		Cc:        []string{"mjl+cc@mox.example", "mjl cc2 <mjl+cc2@mox.example>"},
-		Bcc:       []string{"mjl+bcc@mox.example", "mjl bcc2 <mjl+bcc2@mox.example>"},
-		Subject:   "test email",
-		TextBody:  "this is the content\n\ncheers,\nmox",
-		ReplyTo:   "mjl replyto <mjl+replyto@mox.example>",
-		UserAgent: "moxwebmail/dev",
+		From:           "mjl@mox.example",
+		To:             []string{"mjl+to@mox.example", "mjl to2 <mjl+to2@mox.example>"},
+		Cc:             []string{"mjl+cc@mox.example", "mjl cc2 <mjl+cc2@mox.example>"},
+		Bcc:            []string{"mjl+bcc@mox.example", "mjl bcc2 <mjl+bcc2@mox.example>"},
+		Subject:        "test email",
+		TextBody:       "this is the content\n\ncheers,\nmox",
+		ReplyTo:        "mjl replyto <mjl+replyto@mox.example>",
+		UserAgent:      "moxwebmail/dev",
+		DraftMessageID: draftID,
 	})
 	// todo: check delivery of 6 messages to inbox, 1 to sent
 
@@ -421,17 +457,78 @@ func TestAPI(t *testing.T) {
 	tcompare(t, len(l), 0)
 	tcompare(t, full, true)
 	l, full = api.CompleteRecipient(ctx, "cc2")
-	tcompare(t, l, []string{"mjl cc2 <mjl+cc2@mox.example>"})
+	tcompare(t, l, []string{"mjl cc2 <mjl+cc2@mox.example>", "mjl bcc2 <mjl+bcc2@mox.example>"})
 	tcompare(t, full, true)
 
 	// RecipientSecurity
 	resolver := dns.MockResolver{}
-	rs, err := recipientSecurity(ctx, resolver, "mjl@a.mox.example")
+	rs, err := recipientSecurity(ctx, log, resolver, "mjl@a.mox.example")
 	tcompare(t, err, nil)
 	tcompare(t, rs, RecipientSecurity{SecurityResultUnknown, SecurityResultNo, SecurityResultNo, SecurityResultNo, SecurityResultUnknown})
 	err = acc.DB.Insert(ctx, &store.RecipientDomainTLS{Domain: "a.mox.example", STARTTLS: true, RequireTLS: false})
 	tcheck(t, err, "insert recipient domain tls info")
-	rs, err = recipientSecurity(ctx, resolver, "mjl@a.mox.example")
+	rs, err = recipientSecurity(ctx, log, resolver, "mjl@a.mox.example")
 	tcompare(t, err, nil)
 	tcompare(t, rs, RecipientSecurity{SecurityResultYes, SecurityResultNo, SecurityResultNo, SecurityResultNo, SecurityResultNo})
+
+	// Suggesting/adding/removing rulesets.
+
+	testSuggest := func(msgID int64, expListID string, expMsgFrom string) {
+		listID, msgFrom, isRemove, rcptTo, ruleset := api.RulesetSuggestMove(ctx, msgID, inbox.ID, testbox1.ID)
+		tcompare(t, listID, expListID)
+		tcompare(t, msgFrom, expMsgFrom)
+		tcompare(t, isRemove, false)
+		tcompare(t, rcptTo, "mox@other.example")
+		tcompare(t, ruleset == nil, false)
+
+		// Moving in opposite direction doesn't get a suggestion without the rule present.
+		_, _, _, _, rs0 := api.RulesetSuggestMove(ctx, msgID, testbox1.ID, inbox.ID)
+		tcompare(t, rs0 == nil, true)
+
+		api.RulesetAdd(ctx, rcptTo, *ruleset)
+
+		// Ruleset that exists won't get a suggestion again.
+		_, _, _, _, ruleset = api.RulesetSuggestMove(ctx, msgID, inbox.ID, testbox1.ID)
+		tcompare(t, ruleset == nil, true)
+
+		// Moving in oppositive direction, with rule present, gets the suggestion to remove.
+		_, _, _, _, ruleset = api.RulesetSuggestMove(ctx, msgID, testbox1.ID, inbox.ID)
+		tcompare(t, ruleset == nil, false)
+
+		api.RulesetRemove(ctx, rcptTo, *ruleset)
+
+		// If ListID/MsgFrom is marked as never, we won't get a suggestion.
+		api.RulesetMessageNever(ctx, rcptTo, expListID, expMsgFrom, false)
+		_, _, _, _, ruleset = api.RulesetSuggestMove(ctx, msgID, inbox.ID, testbox1.ID)
+		tcompare(t, ruleset == nil, true)
+
+		var n int
+		if expListID != "" {
+			n, err = bstore.QueryDB[store.RulesetNoListID](ctx, acc.DB).Delete()
+		} else {
+			n, err = bstore.QueryDB[store.RulesetNoMsgFrom](ctx, acc.DB).Delete()
+		}
+		tcheck(t, err, "remove never-answer for listid/msgfrom")
+		tcompare(t, n, 1)
+		_, _, _, _, ruleset = api.RulesetSuggestMove(ctx, msgID, inbox.ID, testbox1.ID)
+		tcompare(t, ruleset == nil, false)
+
+		// If Mailbox is marked as never, we won't get a suggestion.
+		api.RulesetMailboxNever(ctx, testbox1.ID, true)
+		_, _, _, _, ruleset = api.RulesetSuggestMove(ctx, msgID, inbox.ID, testbox1.ID)
+		tcompare(t, ruleset == nil, true)
+
+		n, err = bstore.QueryDB[store.RulesetNoMailbox](ctx, acc.DB).Delete()
+		tcheck(t, err, "remove never-answer for mailbox")
+		tcompare(t, n, 1)
+
+	}
+
+	// For MsgFrom.
+	tdeliver(t, acc, inboxText)
+	testSuggest(inboxText.ID, "", "mjl@mox.example")
+
+	// For List-Id.
+	tdeliver(t, acc, inboxHTML)
+	testSuggest(inboxHTML.ID, "list.mox.example", "")
 }
