@@ -180,9 +180,10 @@ type MessageItem struct {
 // for rendering the (contents of the) message. Information from MessageItem is
 // not duplicated.
 type ParsedMessage struct {
-	ID      int64
-	Part    message.Part
-	Headers map[string][]string
+	ID       int64
+	Part     message.Part
+	Headers  map[string][]string
+	ViewMode store.ViewMode
 
 	// Text parts, can be empty.
 	Texts []string
@@ -215,6 +216,8 @@ type EventStart struct {
 	MailboxName          string
 	Mailboxes            []store.Mailbox
 	RejectsMailbox       string
+	Settings             store.Settings
+	AccountPath          string // If nonempty, the path on same host to webaccount interface.
 	Version              string
 }
 
@@ -487,7 +490,7 @@ type ioErr struct {
 
 // serveEvents serves an SSE connection. Authentication is done through a query
 // string parameter "token", a one-time-use token returned by the Token API call.
-func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *http.Request) {
+func serveEvents(ctx context.Context, log mlog.Log, accountPath string, w http.ResponseWriter, r *http.Request) {
 	if r.Method != "GET" {
 		http.Error(w, "405 - method not allowed - use get", http.StatusMethodNotAllowed)
 		return
@@ -615,7 +618,7 @@ func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *ht
 	accConf, _ := acc.Conf()
 	loginAddr, err := smtp.ParseAddress(address)
 	xcheckf(ctx, err, "parsing login address")
-	_, _, dest, err := mox.FindAccount(loginAddr.Localpart, loginAddr.Domain, false)
+	_, _, _, dest, err := mox.LookupAddress(loginAddr.Localpart, loginAddr.Domain, false, false)
 	xcheckf(ctx, err, "looking up destination for login address")
 	loginName := accConf.FullName
 	if dest.FullName != "" {
@@ -640,6 +643,18 @@ func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *ht
 		}
 		addresses = append(addresses, ma)
 	}
+	// User is allowed to send using alias address as message From address. Webmail
+	// will choose it when replying to a message sent to that address.
+	aliasAddrs := map[MessageAddress]bool{}
+	for _, a := range accConf.Aliases {
+		if a.Alias.AllowMsgFrom {
+			ma := MessageAddress{User: a.Alias.LocalpartStr, Domain: a.Alias.Domain}
+			if !aliasAddrs[ma] {
+				addresses = append(addresses, ma)
+			}
+			aliasAddrs[ma] = true
+		}
+	}
 
 	// We implicitly start a query. We use the reqctx for the transaction, because the
 	// transaction is passed to the query, which can be canceled.
@@ -664,6 +679,7 @@ func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *ht
 	}()
 
 	var mbl []store.Mailbox
+	settings := store.Settings{ID: 1}
 
 	// We only take the rlock when getting the tx.
 	acc.WithRLock(func() {
@@ -673,6 +689,9 @@ func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *ht
 
 		mbl, err = bstore.QueryTx[store.Mailbox](qtx).List()
 		xcheckf(ctx, err, "list mailboxes")
+
+		err = qtx.Get(&settings)
+		xcheckf(ctx, err, "get settings")
 	})
 
 	// Find the designated mailbox if a mailbox name is set, or there are no filters at all.
@@ -728,7 +747,7 @@ func serveEvents(ctx context.Context, log mlog.Log, w http.ResponseWriter, r *ht
 	}
 
 	// Write first event, allowing client to fill its UI with mailboxes.
-	start := EventStart{sse.ID, loginAddress, addresses, domainAddressConfigs, mailbox.Name, mbl, accConf.RejectsMailbox, moxvar.Version}
+	start := EventStart{sse.ID, loginAddress, addresses, domainAddressConfigs, mailbox.Name, mbl, accConf.RejectsMailbox, settings, accountPath, moxvar.Version}
 	writer.xsendEvent(ctx, log, "start", start)
 
 	// The goroutine doing the querying will send messages on these channels, which
@@ -1503,21 +1522,21 @@ func queryMessages(ctx context.Context, log mlog.Log, acc *store.Account, tx *bs
 			unread := map[int64]bool{} // Propagated to thread root.
 			if query.Threading == ThreadUnread {
 				for _, mi := range mil {
-					m := mi.Message
-					if m.Seen {
+					mm := mi.Message
+					if mm.Seen {
 						continue
 					}
-					unread[m.ID] = true
-					for _, id := range m.ThreadParentIDs {
+					unread[mm.ID] = true
+					for _, id := range mm.ThreadParentIDs {
 						unread[id] = true
 					}
 				}
 			}
 			for _, mi := range mil {
-				m := mi.Message
+				mm := mi.Message
 				threadRoot := true
-				rootID := m.ID
-				for _, id := range m.ThreadParentIDs {
+				rootID := mm.ID
+				for _, id := range mm.ThreadParentIDs {
 					if _, ok := collapsed[id]; ok {
 						threadRoot = false
 						rootID = id
@@ -1530,6 +1549,12 @@ func queryMessages(ctx context.Context, log mlog.Log, acc *store.Account, tx *bs
 		} else {
 			have++
 		}
+		if pm != nil && len(pm.envelope.From) == 1 {
+			pm.ViewMode, err = fromAddrViewMode(tx, pm.envelope.From[0])
+			if err != nil {
+				return fmt.Errorf("gathering view mode for id %d: %v", m.ID, err)
+			}
+		}
 		mrc <- msgResp{mil: mil, pm: pm}
 		return nil
 	})
