@@ -39,12 +39,13 @@ func TestView(t *testing.T) {
 	defer func() {
 		err := acc.Close()
 		pkglog.Check(err, "closing account")
+		acc.CheckClosed()
 	}()
 
 	api := Webmail{maxMessageSize: 1024 * 1024, cookiePath: "/"}
 
 	respRec := httptest.NewRecorder()
-	reqInfo := requestInfo{"mjl@mox.example", "mjl", "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
+	reqInfo := requestInfo{log, "mjl@mox.example", acc, "", respRec, &http.Request{RemoteAddr: "127.0.0.1:1234"}}
 	ctx := context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	// Prepare loginToken.
@@ -69,7 +70,7 @@ func TestView(t *testing.T) {
 	}
 	sessionToken := store.SessionToken(sct[0])
 
-	reqInfo = requestInfo{"mjl@mox.example", "mjl", sessionToken, respRec, &http.Request{}}
+	reqInfo = requestInfo{log, "mjl@mox.example", acc, sessionToken, respRec, &http.Request{}}
 	ctx = context.WithValue(ctxbg, requestInfoCtxKey, reqInfo)
 
 	api.MailboxCreate(ctx, "Lists/Go/Nuts")
@@ -103,7 +104,7 @@ func TestView(t *testing.T) {
 	// We start an actual HTTP server to easily get a body we can do blocking reads on.
 	// With a httptest.ResponseRecorder, it's a bit more work to parse SSE events as
 	// they come in.
-	server := httptest.NewServer(http.HandlerFunc(Handler(1024*1024, "/webmail/", false)))
+	server := httptest.NewServer(http.HandlerFunc(Handler(1024*1024, "/webmail/", false, "")))
 	defer server.Close()
 
 	serverURL, err := url.Parse(server.URL)
