@@ -39,6 +39,7 @@ import (
 	"github.com/mjl-/mox/moxio"
 	"github.com/mjl-/mox/store"
 	"github.com/mjl-/mox/webauth"
+	"github.com/mjl-/mox/webops"
 )
 
 var pkglog = mlog.New("webmail", nil)
@@ -51,8 +52,9 @@ type ctxKey string
 var requestInfoCtxKey ctxKey = "requestInfo"
 
 type requestInfo struct {
+	Log          mlog.Log
 	LoginAddress string
-	AccountName  string
+	Account      *store.Account // Nil only for methods Login and LoginPrep.
 	SessionToken store.SessionToken
 	Response     http.ResponseWriter
 	Request      *http.Request // For Proto and TLS connection state during message submit.
@@ -77,7 +79,7 @@ var webmailtextHTML []byte
 var webmailtextJS []byte
 
 var (
-	// Similar between ../webmail/webmail.go:/metricSubmission and ../smtpserver/server.go:/metricSubmission
+	// Similar between ../webmail/webmail.go:/metricSubmission and ../smtpserver/server.go:/metricSubmission and ../webapisrv/server.go:/metricSubmission
 	metricSubmission = promauto.NewCounterVec(
 		prometheus.CounterOpts{
 			Name: "mox_webmail_submission_total",
@@ -170,18 +172,18 @@ func serveContentFallback(log mlog.Log, w http.ResponseWriter, r *http.Request,
 
 // Handler returns a handler for the webmail endpoints, customized for the max
 // message size coming from the listener and cookiePath.
-func Handler(maxMessageSize int64, cookiePath string, isForwarded bool) func(w http.ResponseWriter, r *http.Request) {
+func Handler(maxMessageSize int64, cookiePath string, isForwarded bool, accountPath string) func(w http.ResponseWriter, r *http.Request) {
 	sh, err := makeSherpaHandler(maxMessageSize, cookiePath, isForwarded)
 	return func(w http.ResponseWriter, r *http.Request) {
 		if err != nil {
 			http.Error(w, "500 - internal server error - cannot handle requests", http.StatusInternalServerError)
 			return
 		}
-		handle(sh, isForwarded, w, r)
+		handle(sh, isForwarded, accountPath, w, r)
 	}
 }
 
-func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r *http.Request) {
+func handle(apiHandler http.Handler, isForwarded bool, accountPath string, w http.ResponseWriter, r *http.Request) {
 	ctx := r.Context()
 	log := pkglog.WithContext(ctx).With(slog.String("userauth", ""))
 
@@ -189,7 +191,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 	// messages, and all events afterwards. Authenticated through a token in the query
 	// string, which it got from a Token API call.
 	if r.URL.Path == "/events" {
-		serveEvents(ctx, log, w, r)
+		serveEvents(ctx, log, accountPath, w, r)
 		return
 	}
 
@@ -258,7 +260,9 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 	// All other URLs, except the login endpoint require some authentication.
 	if r.URL.Path != "/api/LoginPrep" && r.URL.Path != "/api/Login" {
 		var ok bool
-		accName, sessionToken, loginAddress, ok = webauth.Check(ctx, log, webauth.Accounts, "webmail", isForwarded, w, r, isAPI, isAPI, false)
+		isExport := r.URL.Path == "/export"
+		requireCSRF := isAPI || isExport
+		accName, sessionToken, loginAddress, ok = webauth.Check(ctx, log, webauth.Accounts, "webmail", isForwarded, w, r, isAPI, requireCSRF, isExport)
 		if !ok {
 			// Response has been written already.
 			return
@@ -266,17 +270,38 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 	}
 
 	if isAPI {
-		reqInfo := requestInfo{loginAddress, accName, sessionToken, w, r}
+		var acc *store.Account
+		if accName != "" {
+			log = log.With(slog.String("account", accName))
+			var err error
+			acc, err = store.OpenAccount(log, accName)
+			if err != nil {
+				log.Errorx("open account", err)
+				http.Error(w, "500 - internal server error - error opening account", http.StatusInternalServerError)
+				return
+			}
+			defer func() {
+				err := acc.Close()
+				log.Check(err, "closing account")
+			}()
+		}
+		reqInfo := requestInfo{log, loginAddress, acc, sessionToken, w, r}
 		ctx = context.WithValue(ctx, requestInfoCtxKey, reqInfo)
 		apiHandler.ServeHTTP(w, r.WithContext(ctx))
 		return
 	}
 
 	// We are now expecting the following URLs:
+	// .../export
 	// .../msg/<msgid>/{attachments.zip,parsedmessage.js,raw}
 	// .../msg/<msgid>/{,msg}{text,html,htmlexternal}
 	// .../msg/<msgid>/{view,viewtext,download}/<partid>
 
+	if r.URL.Path == "/export" {
+		webops.Export(log, accName, w, r)
+		return
+	}
+
 	if !strings.HasPrefix(r.URL.Path, "/msg/") {
 		http.NotFound(w, r)
 		return
@@ -377,7 +402,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 	// allowed. Used to display a message including header. The header is rendered with
 	// javascript, the content is rendered in a separate iframe with a CSP that doesn't
 	// have allowSelfScript.
-	headers := func(sameOrigin, allowExternal, allowSelfScript bool) {
+	headers := func(sameOrigin, allowExternal, allowSelfScript, allowSelfImg bool) {
 		// allow-popups is needed to make opening links in new tabs work.
 		sb := "sandbox allow-popups allow-popups-to-escape-sandbox; "
 		if sameOrigin && allowSelfScript {
@@ -394,6 +419,8 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		var csp string
 		if allowExternal {
 			csp = sb + "frame-ancestors 'self'; default-src 'none'; img-src data: http: https: 'unsafe-inline'; style-src 'unsafe-inline' data: http: https:; font-src data: http: https: 'unsafe-inline'; media-src 'unsafe-inline' data: http: https:" + script
+		} else if allowSelfImg {
+			csp = sb + "frame-ancestors 'self'; default-src 'none'; img-src data: 'self'; style-src 'unsafe-inline'" + script
 		} else {
 			csp = sb + "frame-ancestors 'self'; default-src 'none'; img-src data:; style-src 'unsafe-inline'" + script
 		}
@@ -416,7 +443,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		mi, err := messageItem(log, m, &state)
 		xcheckf(ctx, err, "parsing message")
 
-		headers(false, false, false)
+		headers(false, false, false, false)
 		h.Set("Content-Type", "application/zip")
 		h.Set("Cache-Control", "no-store, max-age=0")
 		var subjectSlug string
@@ -536,7 +563,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		// browsers or users would think of executing. We do set the charset if available
 		// on the outer part. If present, we assume it may be relevant for other parts. If
 		// not, there is not much we could do better...
-		headers(false, false, false)
+		headers(false, false, false, false)
 		ct := "text/plain"
 		params := map[string]string{}
 		if charset := p.ContentTypeParams["charset"]; charset != "" {
@@ -571,7 +598,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		sameorigin := true
 		loadExternal := t[1] == "msghtmlexternal"
 		allowSelfScript := true
-		headers(sameorigin, loadExternal, allowSelfScript)
+		headers(sameorigin, loadExternal, allowSelfScript, false)
 		h.Set("Content-Type", "text/html; charset=utf-8")
 		h.Set("Cache-Control", "no-store, max-age=0")
 
@@ -604,7 +631,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		mijson, err := json.Marshal(mi)
 		xcheckf(ctx, err, "marshal messageitem")
 
-		headers(false, false, false)
+		headers(false, false, false, false)
 		h.Set("Content-Type", "application/javascript; charset=utf-8")
 		h.Set("Cache-Control", "no-store, max-age=0")
 
@@ -636,7 +663,8 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 		// Needed for inner document height for outer iframe height in separate message view.
 		sameorigin := true
 		allowSelfScript := true
-		headers(sameorigin, false, allowSelfScript)
+		allowSelfImg := true
+		headers(sameorigin, false, allowSelfScript, allowSelfImg)
 		h.Set("Content-Type", "text/html; charset=utf-8")
 		h.Set("Cache-Control", "no-store, max-age=0")
 
@@ -662,7 +690,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 			// inner height so we load it as different origin, which should be safer.
 			sameorigin := r.URL.Query().Get("sameorigin") == "true"
 			allowExternal := strings.HasSuffix(t[1], "external")
-			headers(sameorigin, allowExternal, false)
+			headers(sameorigin, allowExternal, false, false)
 
 			h.Set("Content-Type", "text/html; charset=utf-8")
 			h.Set("Cache-Control", "no-store, max-age=0")
@@ -724,7 +752,7 @@ func handle(apiHandler http.Handler, isForwarded bool, w http.ResponseWriter, r
 			ap = ap.Parts[int(index)]
 		}
 
-		headers(false, false, false)
+		headers(false, false, false, false)
 		var ct string
 		if t[1] == "viewtext" {
 			ct = "text/plain"
